(function ($, mf, localizedVars) {
	"use strict";
	const mfSingle = {
		formID: 0,
		lastFieldID: '',
		lastActionID: '',
		/**
	   * Function to initialize all necessary actions associated with current object.
	   */
		init: function () {
			// Define Form ID && Last field ID
			this.formID = this.getFormID();
			this.lastFieldID = this.getLastFieldID(this.formID);
			this.lastActionID = this.getLastActionID();

			// Initialize jQuery UI for specific elements on current page
			this.initContainersjQueryUI();
			this.initFieldsjQueryUI();
			this.initActionsjQueryUI();

			// Initialize event listeners (controling all the different action across editor view, eg: click events)
			this.initGeneralEvents();

			// Initialize field event listeners (listen and make real-time changes on the preview area)
			this.initFieldsEvents();
			this.initActionsEvents();

			// Complete initialization
			this.initComplete();


		},
		/**
	   * These functions is responsible for initializeing the jQuery UI widgets necessary for the rendering of current page.
	   */
		initContainersjQueryUI: function () {

			// Allow wrappers to be sorted
			$("#mfields_holder").sortable({
				tolerance: 'pointer',
				items: '.mf_wrapper',
				delay: 50,
				cursor: 'move',
				revert: false,
				scroll: false,
				scrollSensitivity: 50,
				forcePlaceholderSize: true,
				opacity: '0.9',
				placeholder: 'mf-sortable-placeholder',
				// containment: '#mega-body-fields',
				start: function (event, ui) {
					ui.item.addClass('mf-wrapper-sortable-active');
				},
				stop: function (event, ui) {
					ui.item.removeClass('mf-wrapper-sortable-active');
				},
				cancel: '.disable-sorting',
			}).disableSelection();

			// Allow columns to be sortable so that we can sort and drag fields around
			$(".mf_col").sortable({
				items: '.mfield',
				connectWith: '.mf_col',
				delay: 50,
				cursor: 'move',
				revert: false,
				scroll: false,
				scrollSensitivity: 50,
				opacity: '0.9',
				tolerance: 'pointer',
				over: function (event, ui) {
					mfSingle.checkFieldsEmptyCols();
				},
				out: function (event, ui) {
					mfSingle.checkFieldsEmptyCols();
				},
				stop: function (event, ui) {
					mfSingle.checkFieldsEmptyCols();
				},
				update: function (e, ui) {


					if ($(ui.item).hasClass('ui-draggable')) {


						let type = ui.item.attr('data-type');
						if (typeof type == 'undefined') {
							ui.item.remove();
							return;
						}
						// Call and set the context of `addField()` function to `ui.item` ( ui.item will convert to $(this) in the function context )
						mfSingle.addField.call(ui.item, e, ui.item);
					}
				},
				cancel: '.disable-sorting',
			});

		},
		initFieldsjQueryUI: function (firstLoad = true, type = 'all') {

			// Loop through available field panels and initialize the necessary widgets for each one
			$('.single_field_panel').find('.single_field').each(function () {

				let elementID = $(this).attr('id'),
					editorSelector = $("#" + elementID + "");
				// Initialize the tabs for field editors
				if (editorSelector.length) {

					let field_tab = $(editorSelector).find('.field_options');
					if (!$(field_tab).is('.ui-tabs')) {
						$(field_tab).tabs({ active: 0 });
					}

					// Allow sorting choices on choice fields ( radios, checkboxes, select)
					if ((type == 'radios' || type == 'checkboxes' || type == 'select') || type == 'all') {

						let choices_list = $(editorSelector).find('.mf-choices-list');
						if (!$(choices_list).is('.ui-sortable') || type == 'all') {
							$(choices_list).sortable({
								items: 'li',
								axis: 'y',
								handle: '.sort',
								scroll: false,
								stop: function (e, ui) {
									// Assign the new choice index based on the new order ( to make sure they're saved in the current order )
									$(e.target).find("li").each(function (index) {
										let i = index + 1,
											currentName = $(this).find('input').attr('name'),
											currentNameFragment =  currentName.split("[choices][")[1],
											currentIndex, find, replace;

										currentIndex = currentNameFragment.substr(0, currentNameFragment.indexOf(']')); 
										find = '[choices]['+ currentIndex +']';
										replace = '[choices]['+ i +']';

										$(this).attr('data-key', i);
										$(this).find('input').each(function () {
											$(this).attr('name', $(this).attr('name').replace(find, replace));
										})

									});
									// Refresh the previw area
									mfSingle.refreshFieldPreview(e, e.target, { target: 'none', action: "update_choices" });
								}
							});
						}
					}
				}
			});

			// Only trigger these UI widgets on first load
			if (firstLoad) {
				// assign tabs and accordions
				$('.fields_list_panel').accordion({
					heightStyle: "content",
					active: 0,
					collapsible: true,
				});

				// Allow buttons to be dragged from form buttons container to the columns
				$('.mg_field').draggable({
					helper: "clone",
					cursor: 'move',
					opacity: '0.9',
					scroll: false,
					// cancel: false,
					scrollSensitivity: 50,
					connectToSortable: '.mf_col',
					tolerance: 'pointer',
				});

			}
		},
		initActionsjQueryUI: function (firstLoad = true) {

			if (firstLoad) {
				// assign tabs and accordions
				$('#megaforms_actions').accordion({
					heightStyle: "content",
					active: 0,
					collapsible: true,
				});

				// Allow buttons to be dragged from form buttons container to the columns
				$('.mg_action').draggable({
					helper: "clone",
					cursor: 'move',
					opacity: '0.9',
					scroll: false,
					// cancel: false,
					scrollSensitivity: 50,
					connectToSortable: '.mf_action_col',
					tolerance: 'pointer',
				});
				// Allow columns to be sortable so that we can sort and drag actions around
				$(".mf_action_col").sortable({
					items: '.mfaction',
					handle: '.action_controls',
					delay: 50,
					cursor: 'move',
					revert: false,
					scroll: false,
					scrollSensitivity: 50,
					opacity: '0.9',
					tolerance: 'pointer',
					placeholder: 'mf_action_sortable_placeholder',
					over: function (event, ui) {
						mfSingle.checkActionsEmptyCol();
					},
					out: function (event, ui) {
						mfSingle.checkActionsEmptyCol();
					},
					stop: function (event, ui) {
						mfSingle.checkActionsEmptyCol();
					},
					update: function (e, ui) {
						if ($(ui.item).hasClass('ui-draggable')) {

							let type = ui.item.attr('data-type');
							if (typeof type == 'undefined') {
								ui.item.remove();
								return;
							}
							// Call and set the context of `addAction()` function to `source`  ( ui.item will convert to $(this) in the function context )
							mfSingle.addAction.call(ui.item, e, ui.item);
						}
					},
					cancel: '.disable-sorting',
				});
			}

		},
		/**
		 * This functions is responsible for initializeing the event listeners associated of different compopnents of current page.
		 */
		initGeneralEvents: function () {
			// Toggle fields and field options
			$('.toggle-panels').find('button').click(this.togglePanels);
			// Switch full screen mode on and off
			$("#fullscreenbtn").click(this.switchFullScreen);
			// Save form when "Save" button is clicked
			$("#mform_save").click(this.saveForm);
			// Trigger pop-up when embed icon located at the left of save button is clicked
			$("#mform_embed").click(this.embedForm);
			// Trigger pop-up when tag icon located at the left of form name is clicked
			$("#mgform_title").click(this.renameForm);
			// Add clicked field to fields holder
			$('#megaforms_fields').find(".mg_field").click(this.addField);
			// Trigger container creation
			$('#mega-body-fields').find(".mf_add_container").click(this.addContainer);
			// Add clicked field to fields holder
			$('#megaforms_actions').find(".mg_action").click(this.addAction);
			// Display selected view
			$(".mf-panel-toggler").click(this.changeDisplay);
			// Toggle settings tooltip
			$("#mega-body-settings").find(".settings-tooltip").click(this.showTooltip);
			// Display switch button associated elements on settings
			$("#mega-body-settings").find('[data-toggle-show]').click(this.toggleAssociatedElements);

			// Handle fields selection
			$('#mfields_holder').on("click", ".single_field", function (e) {
				e.stopPropagation();
				mfSingle.selectField(e);
			});
			// Handle fields unselection (unselect any selected field)
			$("#mega-body-fields").on("click", function (e) {
				mfSingle.unselectField(e);
			});
			// Handle container controls
			$('#mfields_holder').on("click", ".container_control", function (e) {
				mfSingle.handleContainerControls(e);
			});
			// Handle field controls
			$('#mfields_holder').on("click", ".field_control", function (e) {
				mfSingle.handleFieldControls(e);
			});
			// Handle field options tooltip
			$('#mform_fields').on("click", ".field-tooltip", function (e) {
				mfSingle.showTooltip(e);
			});
			// Handle action selection
			$('#mfactions_holder').on("click", ".single_action .action_controls", function (e) {
				e.stopPropagation();
				mfSingle.selectAction(e);
			});
			// Handle action controls
			$('#mfactions_holder').on("click", ".action_control", function (e) {
				mfSingle.handleActionControls(e);
			});
			// Handle action controls
			$('#mfactions_holder').on("click", ".action-tooltip", function (e) {
				mfSingle.showTooltip(e);
			});

			$('#mega-forms').on("click", ".mf-inline-modal-btn", function (e) {
				mfSingle.toggleInlineModal(e);
			});

		},
		/**
	   * Listen to field options changes and update value, classes ...etc.
	   */
		initFieldsEvents: function () {

			// Handle real-time preview and fields conditional logic on typing or changing the values of fields
			$('form#mform_fields').change(mfSingle.handleFieldEdits);
			$('form#mform_fields').keypress(mfSingle.handleFieldEdits);
			$('form#mform_fields').keyup(mfSingle.handleFieldEdits);
			$('form#mform_fields').on('click', 'button', function (e) { // to be replaced with conditional logic
				e.preventDefault();
				mfSingle.handleFieldEdits(e, false);
			});

		},
		/**
	   * Listen to action options changes and update value, classes ...etc.
	   */
		initActionsEvents: function () {

			// Nothing yet
			// Handle real-time conditional logic on changes to the actions form
			// $('form#mform_actions').change(mfSingle.handleFieldEdits); // to be replaced with conditional logic

		},
		/**
	   * Complete initialization.
	   */
		initComplete: function (e) {

			// Make sure any empty columns has the correct css class
			mfSingle.checkFieldsEmptyCols();
			mfSingle.checkActionsEmptyCol();
			// Initialize conditional fields
			mf.conditionalFields('#mform_fields, #mform_actions, #mform_settings', 'inline', true);
			// Toggle pre-defined options
			// mfSingle.toggleAssociatedElements(undefined, 'input[name=limited_entries]'); // For settings
			// mfSingle.toggleAssociatedElements(undefined, 'input[name=login_restricted]'); // For settings
			// mfSingle.toggleAssociatedElements(undefined, 'input[name=confirmation_type]'); // For settings
			// // Toggle For pre-loaded fields
			// $('.single_field_panel').find('.single_field').each(function (e) {
			// 	let el = $(this).find('[data-toggle-parent]');
			// 	mfSingle.toggleAssociatedElements(e, el);
			// });

			// Remove initilization class to allow some element to display after initilization
			$('.initializing').removeClass("initializing");
		},
		/**
	   * Function run all the necessary screen resize processes
	   */
		onResize: function () {

			// Do not allow mega forms container to take more than full page size .
			// ( usefull for large screens since the container size is set based on viewport size ).
			let pageHeight = $(document).height();

			if (!$('body').hasClass('fullscreen')) {
				let containerSize = pageHeight - 215;
				$('.mega-body-content').css("max-height", containerSize + "px");
				$('.mega-container').css("max-height", containerSize + "px");
			}
		},
		/**
	   * Event callbacks.
	   */
		togglePanels: function (e, tab = false) {
			e.preventDefault();
			// if `set` is defined select the supplied tab
			if (tab !== false) {
				let singleFieldPanel = $('.single_field_panel'),
					singleFieldPanelBtn = $('.edit-form__single-field-tab'),
					allFieldsPanel = $('.fields_list_panel'),
					allFieldsPanelBtn = $('.edit-form__fields-tab');
				if (tab == 'editor') {
					// Set and enable current field settings
					let current = $(singleFieldPanelBtn).attr('data-current');
					$(singleFieldPanel).find('.is-active').removeClass('is-active');
					$(singleFieldPanel).find('[data-id="' + current + '"]').addClass('is-active');
					// Switch from `All Fields` to `Edit Field`
					$(allFieldsPanelBtn).removeClass("is-active");
					$(singleFieldPanelBtn).addClass("is-active");
					$(allFieldsPanel).removeClass("is-active");
					$(singleFieldPanel).addClass("is-active");

				} else {
					// unselect any selected field
					$('.toggle-panels').find('.edit-form__single-field-tab').attr('data-current', 'empty');
					$('#mfields_holder').find('.selected').removeClass("selected");
					// Switch from `Edit Field` to `All Fields`
					$(singleFieldPanel).find('.is-active .tabs:first-child a').trigger('click'); // Make sure to return to the first tab before unselecting field
					$(singleFieldPanelBtn).removeClass("is-active");
					$(allFieldsPanelBtn).addClass("is-active");
					$(singleFieldPanel).removeClass("is-active");
					$(allFieldsPanel).addClass("is-active");


				}
				return;
			}

			// Just Toggle if `tab` define it and re-call current function
			if (!$(this).hasClass('is-active')) {
				if ($(this).hasClass('edit-form__single-field-tab')) {
					mfSingle.togglePanels(e, 'editor');
				} else {
					mfSingle.togglePanels(e, 'picker');
				}
			}
		},
		selectField: function (e) {
			e.preventDefault();
			let el = e.currentTarget;
			if (!$(el).hasClass('selected')) {

				// Get field ID
				let fieldID = $(el).data('id');
				// Select clicked field
				$('#mfields_holder').find('.selected').removeClass("selected");
				$(el).addClass("selected");
				// Set current active tab (left)
				$('.toggle-panels').find('.edit-form__single-field-tab').attr('data-current', fieldID);

				// Toggle field editor
				mfSingle.togglePanels(e, 'editor');
			}
		},
		unselectField: function (e) {
			// See if one of the fields is clicked
			if ($(e.target).hasClass('.single_field')) {
				return;
			}
			// Ignore clicks on .single_field children
			if ($(e.target).closest('.single_field').length) {
				return;
			}

			e.preventDefault();
			if ($('.edit-form__single-field-tab').hasClass('is-active')) {
				// Switch from  `Edit Field` to `All Fields`
				mfSingle.togglePanels(e, 'picker');
			}
		},
		selectAction: function (e) {

			if ($(e.target).hasClass('.action_control') || $(e.target).parents('.action_control').length) {
				return;
			}

			e.preventDefault();
			let controls = e.currentTarget,
				actionWrapper = $(controls).closest('.single_action');

			if (!$(actionWrapper).hasClass('selected')) {

				// Select clicked field
				$('#mfactions_holder').find('.single_action.selected').removeClass("selected");
				$(actionWrapper).addClass("selected");

			} else {
				$(actionWrapper).removeClass("selected");
			}
		},
		switchFullScreen: function (e) {
			e.preventDefault();
			$(window).scrollTop(0);
			$('.wp-toolbar').stop().toggleClass('fullscreen');
			$('body').stop().toggleClass('fullscreen');
			$('#wpcontent').stop().toggleClass('fullscreen');
			$('#mega-forms').stop().toggleClass('fullscreen');
			$('#mega-forms-container').stop().toggleClass('fullscreen');
		},
		addField: function (e, item = false) {
			// Define needed data
			let type = $(this).attr('data-type'),
				field_id = mfSingle.incrementFieldID(),
				icon = $(this).attr('icon'),
				title = $(this).attr('value');

			// Get field preview template (placeholder while field is being retrieved from server side)
			let fieldPreviewTemplate = wp.template('single_field'),
				fieldPreviewOutput = fieldPreviewTemplate({
					title: title,
					field_id: field_id,
					icon: icon,
					type: type,
					spinner: localizedVars.spinner,
				});

			// If the item to be replaced is not defined, add a loader to the lastest column
			if (item === false) {
				$('#mfields_holder').find('.mf_wrapper').last().find('.mf_col').last().removeClass('mf-empty-col').append(fieldPreviewOutput);
			} else {
				$(item).replaceWith(fieldPreviewOutput);
			}
			// Retrieve the field via Ajax and replace the placeholder
			mfSingle.getField(type, field_id, title);

		},
		addAction: function (e, item = false) {
			// Define needed data
			let type = $(this).attr('data-type'),
				action_id = mfSingle.incrementActionID(),
				title = $(this).attr('value');
			// Get action preview template (placeholder while action is being retrieved from server side)
			let actionPreviewTemplate = wp.template('single_action'),
				actionPreviewOutput = actionPreviewTemplate({
					title: title,
					action_id: action_id,
					type: type,
					spinner: localizedVars.spinner,
				});

			// If the item to be replaced is not defined, add a loader to the lastest column
			if (item === false) {
				$('#mfactions_holder').find('.mf_action_col').removeClass('mf-empty-col').append(actionPreviewOutput);
			} else {
				$(item).replaceWith(actionPreviewOutput);
			}
			// Retrieve the field via Ajax and replace the placeholder
			mfSingle.getAction(type, action_id, title);

		},
		addContainer: function (e) {
			// Get field preview template (placeholder while field is being retrieved from server side)
			let containerTemplate = wp.template('single_container'),
				containerOutput = containerTemplate({});

			$('#mfields_holder').append(containerOutput);
			mfSingle.checkFieldsEmptyCols();

			// Make the new items recognizable to accept sorting/drag/drop
			mfSingle.initContainersjQueryUI();
		},
		renameForm: function (e) {
			e.preventDefault();
			let modalTemplate = wp.template('rename-form-modal'),
				modalOutput = modalTemplate({
					heading: localizedVars.labels.renameForm,
					desc: localizedVars.text.RenameFormText,
					label: localizedVars.labels.renameFormEg,
					submittext: localizedVars.labels.rename,
					spinner: localizedVars.spinner,
				});

			// Add out element to the DOM and add focus to the first input
			$('body').append(modalOutput).find('[data-hook="modal-title-input"]').focus();
			let modal = document.getElementById('mgform_modal');

			// Close Modal on close button click
			$('[data-hook="modal-close"]').click(function (e) {
				e.preventDefault();
				$(modal).remove();
			});

			// Trigger form creation on button click or keypress
			$('[data-hook="modal-title-input"]').keyup(function (event) {
				if (event.keyCode == 13) {
					event.preventDefault();
					let name = $('#new_mgform_title').val();
					if (name !== '') {
						mfSingle.updateFormName(name);
					} else {
						$(this).siblings('.error').text(localizedVars.text.formNameError);
					}

				}
			});

			$('[data-hook="modal-title-submit"]').click(function (e) {
				e.preventDefault();
				let name = $('#new_mgform_title').val();
				if (name !== '') {
					mfSingle.updateFormName(name);
				} else {
					$(this).siblings('.error').text(localizedVars.text.formNameError);
				}
			});

		},
		embedForm: function (e) {
			e.preventDefault();
			let modalTemplate = wp.template('empty-modal'),
				modalOutput = modalTemplate({
					heading: localizedVars.labels.embedForm,
					desc: localizedVars.text.embedFormText,
					content: '<input class="display-shorcode" onclick="this.focus();this.select()" readonly="readonly" value="[megaforms id=' + mfSingle.formID + ']">'
				});
			// Add out element to the DOM and add focus to the first input
			$('body').append(modalOutput);
			let modal = document.getElementById('mgform_modal');

			// Close Modal on close button click
			$('[data-hook="modal-close"]').click(function (e) {
				e.preventDefault();
				$(modal).remove();
			});

		},
		saveForm: function (e) {
			e.preventDefault();

			let containers = mfSingle.getAvailableContainers(),
				fields = mfSingle.convertFormFieldsToArray("form#mform_fields", 'fields'),
				settings = mfSingle.convertFormFieldsToArray("form#mform_settings", 'settings'),
				actions = mfSingle.convertFormFieldsToArray("form#mform_actions", 'actions');

			mfSingle.updateForm(containers, fields, settings, actions);
		},
		showTooltip: function (e) {
			e.preventDefault();

			let el = e.currentTarget,
				content = $(el).find('#tooltip-content'),
				offest = 0;

			if ($(el).hasClass('field-tooltip')) {
				// Handle fields tooltip
				offest = ($(el).offset().top - $(el).closest('.tabs-panel').offset().top) + 18;
			} else if ($(el).hasClass('action-tooltip')) {
				// Handle actions tooltip
				offest = ($(el).offset().top - $(el).closest('.single-action_panel').offset().top) + 18;
			} else if ($(el).hasClass('settings-tooltip')) {
				// Handle settings tooltip
				offest = ($(el).offset().top - $(el).closest('#mega-body-settings').offset().top) + 18;
			}

			if (offest !== 0) {
				$(content).css({ top: '' + offest + 'px' }).fadeIn();
				$(el).mouseleave(function () {
					$(content).fadeOut();
				});
			}

		},
		toggleInlineModal: function (e) {
			e.preventDefault();

			let el = e.currentTarget,
				container = $(el).closest('.mf_field_container'),
				form = $(el).closest('.mf_field_container').closest('form'),
				templates = $(el).attr('data-templates');


			if ($(el).hasClass('active')) {
				// Handle clicking same button when the modal is open
				$(container).find('#mf_inline_modal').remove();
				$(el).removeClass('active');
				$(form).removeClass('has_active_inline_modal');
			} else {
				// Handle opening a modal when another one is already opened
				if ($(form).hasClass('has_active_inline_modal')) {
					$(form).find('#mf_inline_modal').remove();
					$(form).find('.mf-inline-modal-btn.active').removeClass('active');
					$(form).removeClass('has_active_inline_modal');
				}

				if (typeof templates !== "undefined") {
					templates = templates.split(",");

					let tabs = '',
						content = '';

					for (let i = 0; templates.length > i; i++) {
						let type = templates[i],
							subContent = '';

						// Dynamically add content to the inline modal 'fields' for merge tags
						if (type == 'fields') {
							let fields = mfSingle.getAvailableFields();
							for (let n = 0; fields.length > n; n++) {
								subContent += '<a href="javascript:void(0)" data-tag="{mf:' + type + ' ' + fields[n].id + '}" class="mf_insert_tag">'
								subContent += '<span class="merge-tag-title">' + fields[n].label + '</span>';
								subContent += '<span class="merge-tag-code">{mf:' + type + ' ' + fields[n].id + '}</span>';
								subContent += '</a>';
							}
						}

						let inlineModalContent = wp.template('inline-modal-' + type),
							output = inlineModalContent({
								content: subContent
							}),
							tempEl = null;

						tempEl = document.createElement("div")
						tempEl.innerHTML = output;

						let tabTitle = tempEl.getElementsByClassName('inline-modal-tab-link')[0],
							tabContent = tempEl.getElementsByClassName('inline-modal-content')[0];
						// Hide all tabs content by default except the first one
						if (i === 0) {
							$(tabTitle).addClass('active')
							$(tabContent).addClass('active')
						}

						tabs += tabTitle.outerHTML;
						content += tabContent.outerHTML;

						tempEl = null;

					}
					let inlineModal = wp.template('inline-modal'),
						modalHTML = inlineModal({
							tabs: tabs,
							content: content
						});

					$(container).append(modalHTML);
					let modal = $(container).find('#mf_inline_modal'),
						offest = ($(el).offset().top - $(el).closest('.tabs-panel, .single-action_panel').offset().top) + 28;

					$(modal).css({ top: '' + offest + 'px' }).fadeIn();
					$(el).addClass('active');
					$(form).addClass('has_active_inline_modal');

					// Add merge tags to the input on click
					$(modal).find('.mf_insert_tag').click(function (e) {
						let value = $(this).closest('.mf_field_container').find('input, textarea').val() + $(this).attr('data-tag');
						$(this).closest('.mf_field_container').find('input, textarea').val(value);
					});
					// Handle tabs navigation
					$(modal).find('.inline-modal-tab-link').click(function (e) {
						let id = $(this).attr('data-id');
						$(this).siblings('.active').removeClass('active');
						$(this).addClass('active');
						$(this).closest('#mf_inline_modal').find('.inline-modal-content.active').removeClass('active');
						$(this).closest('#mf_inline_modal').find('.inline-modal-content[data-id="' + id + '"]').addClass('active');
					});

					// remove the modal when mouseout of the whole container
					$(modal).closest('.field_container').mouseleave(function (e) {
						$(container).find('#mf_inline_modal').remove();
						$(el).removeClass('active');
						$(form).removeClass('has_active_inline_modal');
						$(this).unbind(e);
					});
				}
			}

		},
		changeDisplay: function (e) {
			e.preventDefault();

			let el = e.currentTarget,
				panelToDisplay = $(el).data('panel'), // ID based
				panelToHide = $(el).data('disable');	// Class based

			// Hide active panels
			$(el).siblings('.active').removeClass('active');
			$('.' + panelToHide + '.active').removeClass('active');

			// Display the one selected
			$(el).addClass('active');
			$('#' + panelToDisplay + '').addClass('active');
		},
		handleContainerControls: function (e) {

			let el = e.currentTarget,
				container = $(el).closest('.mf_wrapper_preview'),
				action = $(el).data('action');

			if (action !== undefined && action !== '') {

				e.stopImmediatePropagation();
				e.stopPropagation();
				e.preventDefault();

				if (action == 'edit') {
					mfSingle.handleContainerEdit(el, container);
				} else if (action == 'delete') {
					mfSingle.handleContainerDelete(el, container);
				} else if (action == 'layout') {
					mfSingle.handleContainerLayout(el, container);
				}

			}
		},
		handleFieldControls: function (e) {

			let el = e.currentTarget,
				current = $(el).closest('.field_preview').attr('data-id'),
				action = $(el).data('action');

			if (action !== undefined && action !== '') {

				e.stopImmediatePropagation();
				e.stopPropagation();
				e.preventDefault();

				if (action == 'duplicate') {
					mfSingle.handleFieldDuplicate(e, el, current);
				} else if (action == 'delete') {
					mfSingle.handleFieldDelete(e, el, current);
				}

			}
		},
		handleActionControls: function (e) {

			let el = e.currentTarget,
				current = $(el).closest('.single_action').attr('data-id'),
				action = $(el).data('action');

			if (action !== undefined && action !== '') {

				e.stopImmediatePropagation();
				e.stopPropagation();
				e.preventDefault();

				if (action == 'delete') {
					mfSingle.handleActionDelete(e, el, current);
				}

			}
		},
		toggleAssociatedElements: function (e, target = false) {
			let el = target == false ? e.currentTarget : $(target),
				parent, hide, show, val;

			if ($(el).length > 1) { // Find the checked value for radios
				$(el).each(function (e) {
					if ($(this).is(':checked')) {
						parent = $(this).data('toggle-parent');
						hide = $(this).data('toggle-hide');
						show = $(this).data('toggle-show');
						val = $(this).getVal();
					}
				});
			} else {
				parent = $(el).data('toggle-parent');
				hide = $(el).data('toggle-hide');
				show = $(el).data('toggle-show');
				val = $(el).getVal();
			}

			if (parent) {
				if (val) {
					if (hide) {
						$(parent).find(hide).hide();
					}
					$(parent).find(show).show();
				} else {
					$(parent).find(show).hide();
					if (hide) {
						$(parent).find(hide).show();
					}
				}
			}
		},
		/**
	   * AJAX.
	   */

		getField: function (type, field_id, field_label = '') {

			let form_id = mfSingle.formID,
				loader = $("#mfields_holder").find("[data-id='" + field_id + "']");

			let data = {
				type: type,
				form_id: form_id,
				field_id: field_id,
				field_label: field_label,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'get_field',
				success: function (data) {
					$(loader).replaceWith(data.display);
					$('.edit-form-panel-available_fields').append(data.settings);
					mfSingle.initFieldsjQueryUI(false, type);
					mfSingle.checkFieldsEmptyCols();
					mfSingle.triggerConditionsRefresh('fields', 'add');
				},
				fail: function (data) {
					$(loader).remove();
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});

		},
		getAction: function (type, action_id, label = '') {

			let form_id = mfSingle.formID,
				loader = $("#mfactions_holder").find("[data-id='" + action_id + "']");

			let data = {
				type: type,
				form_id: form_id,
				action_id: action_id,
				action_label: label,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'get_action',
				success: function (data) {
					$(loader).replaceWith(data.output);
					mfSingle.checkActionsEmptyCol();
					mfSingle.triggerConditionsRefresh('actions', 'add');
				},
				fail: function (data) {
					$(loader).remove();
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});

		},
		updateFormName: function (name) {
			let response = $('[data-hook="modal-response"]'),
				input = $('[data-hook="modal-title-input"]'),
				submit = $('[data-hook="modal-title-submit"]');

			$(input).prop('disabled', true);
			$(submit).prop('disabled', true);
			$(response).show();

			let data = {
				form_name: name,
				form_id: mfSingle.formID,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'rename_form',
				success: function (data) {
					// Show success message and display new name on current form without refresh
					$(response).find('span img').hide();
					$(response).find('span').append(data.message);
					$('#mgform_switcher').data('name', name);
					$('#mgform_switcher').find('span').text(name);
				},
				fail: function (data) {
					$(input).prop('disabled', false);
					$(submit).prop('disabled', false);
					$(response).hide();
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},
		updateForm: function (containers, fields, settings, actions) {
			$('#saving').show();
			$('#mform_save').addClass('disabled');

			let data = {
				form_id: mfSingle.formID,
				containers: containers,
				fields: fields,
				settings: settings,
				actions: actions,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'update_form',
				success: function (data) {

					$('#mform_save').removeClass('disabled');
					$('#saving').hide();
					// Display sucess message in a snackbar notice
					mfSingle.showSnackbar({ text: data.message, customClass: 'success' });

				},
				fail: function (data) {
					$('#mform_save').removeClass('disabled');
					$('#saving').hide();
					// Display error message in a snackbar notice
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });

				}
			});
		},

		/**
	   * Helper Functions.
	   */

		getFormID: function () {
			// Try the URL
			let FormID = null,
				tmp = [];
			location.search
				.substr(1)
				.split("&")
				.forEach(function (item) {
					tmp = item.split("=");
					if (tmp[0] === 'id') FormID = decodeURIComponent(tmp[1]);
				});

			// Try Form data
			if (FormID == null || FormID == '' || isNaN(FormID)) {
				FormID = $('#mform_fields').attr("data-id");
			}
			return parseInt(FormID);
		},
		getLastFieldID: function (formID) {
			// Set first part (FORM ID)
			let fieldID = 'mf_';
			// Set Second part (String = _field_ )
			fieldID += formID;
			fieldID += '_field_';
			// Set Third part ( Get latest field ID )
			let latest_id = $('#mform_fields').attr("data-last-field-id");
			let id = 0;
			if (typeof (latest_id) !== undefined && latest_id > 0) {
				id = parseFloat(latest_id); // Use this as a base ID
			}

			$('#mfields_holder').find(".single_field").each(function () {
				let currentID = parseFloat($(this).attr('data-id')),
					itemID = (currentID > 0) ? currentID : 0;
				if (itemID > id) {
					id = itemID;
				}
			});

			fieldID += parseFloat(id);

			return fieldID;
		},
		getLastActionID: function () {

			// Get latest action ID
			let id = 0;

			$('#mfactions_holder').find(".single_action").each(function () {
				let currentID = parseFloat($(this).attr('data-id')),
					itemID = (currentID > 0) ? currentID : 0;
				if (itemID > id) {
					id = itemID;
				}
			});

			return parseInt(id);
		},
		incrementFieldID: function () {
			let id = mfSingle.lastFieldID;
			// Get the index of field ID
			let split = id.lastIndexOf('_');
			// Use the index to get the field number and Increment it by one
			let newFieldNumber = parseInt(id.substring(split + 1)) + 1;
			// Recreate field ID by getting first part of the previous ID and append the new field number to id.
			let newID = id.substring(0, split);
			newID += '_';
			newID += newFieldNumber;

			// Set last field ID before sending back the new ID
			mfSingle.lastFieldID = newID;
			// Return new field number
			return newFieldNumber;
		},
		incrementActionID: function () {
			let id = mfSingle.lastActionID;

			// Use the index to get the action number and Increment it by one
			let newActionNumber = parseInt(id) + 1;
			// Set last action ID before sending back the new ID
			mfSingle.lastActionID = newActionNumber;
			// Return new field number
			return newActionNumber;
		},
		getAvailableFields: function () {
			let fields = [];
			$('.edit-form-panel-available_fields').find('li.single_field').each(function () {
				if ($(this).attr('data-is-static') == '0') {
					let field = {
						id: $(this).attr('data-id'),
						type: $(this).attr('data-type'),
						label: $(this).find('.mf-inner-field_label input').val(),
					};
					fields.push(field);
				}
			});
			return fields;
		},
		handleFieldEdits: function (e, useTarget = true) {
			// $target is the element that triggered the event (e.g., the user clicked on)
			// $currentTarget is the element that the event listener is attached to.
			let element = useTarget ? e.target : e.currentTarget;
			if (typeof $(element).data('preview') !== 'undefined') {

				let preview = $(element).data('preview');
				if (typeof preview === 'object' && typeof preview.target !== 'undefined' && typeof preview.action !== 'undefined') {
					mfSingle.refreshFieldPreview(e, element, preview);
				}

			} else if (typeof $(element).data('edit') !== 'undefined') {

				let edit = $(element).data('edit');
				if (typeof edit === 'object' && typeof edit.target !== 'undefined' && typeof edit.action !== 'undefined') {
					mfSingle.refreshFieldEditor(e, element, edit);
				}

			}

			return;

		},
		refreshFieldPreview: function (e, element, preview) {
			let currentField = $(element).closest('.field_container').data('id'),
				targetParent = $('#mega-body-fields').find('#mf_' + mfSingle.formID + '_field_' + currentField + ''),
				target = preview.target === 'none' ? $(targetParent) : $(targetParent).find(preview.target),
				action = preview.action,
				value = $(element).getVal(),
				args = 'args' in preview ? preview.args : {};

			console.log(action);
			console.log(target);
			console.log(value);

			if (target.length > 0) {

				switch (action) {
					case 'update_label':
						$(target).html(value);
						break;
					case 'update_sub_label':
						/*
						* Helper data attributes (data-default = fallback value)
						*/
						if (value == '') {
							value = $(element).attr('data-default');
							value = value == undefined ? '' : value;
						}
						$(target).find('.mf_sub_label').html(value);
						break;
					case 'update_placeholder':
						/*
						* Helper data attributes (data-default = fallback value)
						*/
						if ($(target).is('select')) {
							if (value == '') {
								value = $(element).attr('data-default');
								value = value == undefined ? '' : value;
							}
							$(target).find('.mf_select_placeholder').text(value);
						} else {
							$(target).attr('placeholder', value);
						}
						break;
					case 'update_value':
						$(target).val(value);
						break;
					case 'update_desc':
						if (value !== '') {
							$(target).html(value);
						} else {
							$(target).remove();
						}
						break;
					case 'update_required':
						if (!value) {
							$(target).remove();
							$(targetParent).removeClass('mfield_required ');
						}
						break;
					case 'update_descPosition':

						var mf_desc_clone = $(target).clone().addClass('mf_hidden');
						$(target).remove();

						if (value == 'top') {
							$(mf_desc_clone).removeClass('mf_hidden');
							$(targetParent).find('.mf_input_container').before(mf_desc_clone);
						} else if (value == 'bottom') {
							$(mf_desc_clone).removeClass('mf_hidden');
							$(targetParent).find('.mf_input_container').after(mf_desc_clone).removeClass('mf_hidden');
						} else {
							$(targetParent).find('.mf_input_container').after(mf_desc_clone);
						}

						break;
					case 'update_subLabelPosition':
						$(targetParent).find('.mf_sub_field').each(function (e) {
							if (value == 'hidden') {
								$(this).find('.mf_sub_label').addClass('mf_hidden');
							} else {
								let subLabel = $(this).find('.mf_sub_label'),
									subLabel_clone = $(subLabel).clone().removeClass('mf_hidden');
								$(subLabel).remove();
								if (value == 'top') {
									$(this).prepend(subLabel_clone);
								} else {
									$(this).append(subLabel_clone);
								}
							}
						});
						break;
					case 'update_choices':

						var choiceType = $(targetParent).attr('data-type');
						var existingChoices = [];

						$(element).closest('.mf-choices-list').find('li').each(function () {
							let choice = {};

							choice['label'] = $(this).find('.label').val();
							choice['value'] = $(this).find('.label').val();
							choice['default'] = $(this).find('.default').getVal();

							existingChoices.push(choice);
						});
						// Handle select fields
						if (choiceType == 'select') {

							let selectOption = $(targetParent).find('select').find('option').first(),
								newOptions = '';
							$.each(existingChoices, function (i, obj) {
								let newSelectOption = $(selectOption).clone();
								$(newSelectOption).val(obj.value).text(obj.label);
								if (obj.default) {
									$(newSelectOption).attr('selected', 'selected');
								} else {
									$(newSelectOption).removeAttr('selected');
								}
								newOptions += $(newSelectOption)[0].outerHTML;
							});
							$(targetParent).find('select').empty().append(newOptions);

						} else { // Handle radios and checkboxes

							let choiceOptions = $(targetParent).find('.mf-choice').find('li').first(),
								newChoices = '';
							$.each(existingChoices, function (i, obj) {

								let newSelectChoices = $(choiceOptions).clone(),
									newSelectChoicesInput = $(newSelectChoices).find('input');
								$(newSelectChoicesInput).val(obj.value);
								if (obj.default) {
									$(newSelectChoicesInput).attr('checked', 'checked');
								} else {
									$(newSelectChoicesInput).removeAttr('checked');
								}
								$(newSelectChoices).find('.mf-choice-desc').text(obj.label);
								newChoices += $(newSelectChoices)[0].outerHTML;
							});

							$(targetParent).find('.mf-choice').empty().append(newChoices);
						}

						break;
					case 'toggle_field_inputs':
						var startDirection = 'left';

						if (value) {
							$(target).removeClass('mf_hidden');
						} else {
							$(target).addClass('mf_hidden');
						}

						$(targetParent).find('.mf_half').removeClass('left right');
						$(targetParent).find('.mf_half').each(function (e) {
							if (!$(this).hasClass('mf_hidden')) {
								$(this).addClass(startDirection);
								startDirection = startDirection == 'left' ? 'right' : 'left';
							}
						});
						break;
					case 'update_divider_styles':
						var style = $(element).attr('data-style'),
							suffix = 'px';

						if (value < 5 && (style == 'margin-top' || style == 'margin-bottom')) {
							value = 5;
						}

						if (style == 'width') {
							suffix = '%';
						}

						$(target).css(style, value + suffix);
						break;
					case 'update_size':
						if (value == 'half') {
							$(target).removeClass('mf_full').addClass('mf_half');
						} else if (value == 'full') {
							$(target).removeClass('mf_half').addClass('mf_full');
						}
						break;
					case 'add_class':
						/*
						* Helper data attributes (data-add-class = The class that need to be added)
						*/
						var classToAdd = $(element).attr('data-add-class');
						if (typeof classToAdd !== 'undefined') {
							$(target).addClass(classToAdd);
						}
						break;
					case 'remove_class':
						/*
						* Helper data attributes (data-remove-class = The class that need to be removed)
						*/
						var classToRemove = $(element).attr('data-remove-class');
						if (classToRemove) {
							$(target).removeClass(classToRemove);
						}
						break;
					case 'change_border_color':
						/*
						* Change the element background color
						*/
						if ('color' in args) {

							if (!value) {
								$(target).css('border-color', '');
							} else {
								$(target).css('border-color', args.color);
							}

						}

						break;
				}

			} else {

				mfSingle.insertPreDefinedNodes(e, element, action, targetParent, value, currentField, preview.target);

			}
		},
		refreshFieldEditor: function (e, element, edit) {
			let currentField = $(element).closest('.field_container'),
				target = edit.target === 'none' ? $(currentField) : $(currentField).find(edit.target),
				action = edit.action,
				value = $(element).getVal(),
				args = 'args' in edit ? edit.args : {};

			if (target.length > 0) {
				switch (action) {
					case 'show_choices_values':
						if (value) {
							$(target).addClass('mf-choices-values-active');
						} else {
							$(target).removeClass('mf-choices-values-active');
						}
						break;
					case 'add_choice':
						var choice = $(element).parent('li'),
							choiceID = $(choice).attr('data-key'),
							choiceDefaultname = $(choice).find('.default').attr('name'),
							choiceLabelname = $(choice).find('.label').attr('name'),
							choiceValuename = $(choice).find('.value').attr('name'),
							choiceClone = $(choice).clone();

						// Get latest ID
						var id = 0;
						$(element).closest('.mf-choices-list').find('li').each(function () {
							let currentID = parseFloat($(this).attr('data-key')),
								itemID = (currentID > 0) ? currentID : 0;
							if (itemID > id) {
								id = itemID;
							}
						});
						// Create new ID and apply to cloned inputs
						var newChoiceID = parseFloat(id) + 1,
							newChoiceDefaultname = choiceDefaultname.replace("[choices][" + choiceID + "]", "[choices][" + newChoiceID + "]"),
							newChoiceLabelname = choiceLabelname.replace("[choices][" + choiceID + "]", "[choices][" + newChoiceID + "]"),
							newChoiceValuename = choiceValuename.replace("[choices][" + choiceID + "]", "[choices][" + newChoiceID + "]");

						var cloneDefault = $(choiceClone).find('.default'),
							cloneLabel = $(choiceClone).find('.label'),
							cloneValue = $(choiceClone).find('.value');


						$(cloneDefault).prop('checked', false);
						$(cloneLabel).val('');
						$(cloneValue).val('');
						$(cloneDefault).attr('name', newChoiceDefaultname);
						$(cloneLabel).attr('name', newChoiceLabelname);
						$(cloneValue).attr('name', newChoiceValuename);
						$(choiceClone).attr('data-key', newChoiceID);

						$(choiceClone).insertAfter(choice);
						// Update choices in the preview
						mfSingle.refreshFieldPreview(e, element, { target: edit.target, action: "update_choices" });

						break;
					case 'remove_choice':
						var availableChoices = $(element).closest('.mf-choices-list').find('li');

						if (availableChoices.length === 1) {
							mfSingle.showSnackbar({ text: localizedVars.text.removeChoiceN, customClass: 'warning' });
						} else {
							let backupElement = $(element).parent('li').siblings('li').find('.remove');
							$(element).parent('li').remove();
							// Update choices in the preview
							mfSingle.refreshFieldPreview(e, backupElement, { target: edit.target, action: "update_choices" });
						}

						break;
					case 'select_default_choice':

						if ($(element).attr('type') == 'radio') {
							$(element).closest('.mf-choices-list').find('li').each(function () {
								$(this).find('.default').prop('checked', false);
							});

							$(element).prop('checked', true);
						}

						// Update choices in the preview
						mfSingle.refreshFieldPreview(e, element, { target: edit.target, action: "update_choices" });

						break;
					case 'change_choice_label':

						var choiceValEl = $(element).next('.value'),
							choiceVal = $(choiceValEl).val();
						if (element.defaultValue === choiceVal || $(element).attr("data-prev-val") === choiceVal || "" == choiceVal) {
							$(choiceValEl).val(value);
							$(element).attr("data-prev-val", value);
						}

						// Update choices in the preview
						mfSingle.refreshFieldPreview(e, element, { target: edit.target, action: "update_choices" });

						break;
					case 'navigate_input_settings':
						/*
						* Helper data attributes (data-type = A css class defining the elements that will be displayed)
						*/
						var activeInput = $(element).siblings('.active'),
							newActiveInput = null;

						if ($(element).hasClass('next')) {

							let nextEl = $(activeInput).next('span');
							if (nextEl.length === 0) {
								nextEl = $(activeInput).siblings('.first');
							}
							newActiveInput = nextEl;
						} else {

							let prevEl = $(activeInput).prev('span');
							if (prevEl.length === 0) {
								prevEl = $(activeInput).siblings('.last');
							}
							newActiveInput = prevEl;
						}

						$(activeInput).removeClass('active');
						$(newActiveInput).addClass('active');

						var selectedType = $(newActiveInput).attr('data-type');

						$('.mf-field-inputs').find('.active').removeClass('active');
						$('.mf-field-inputs').find('.' + selectedType + '').addClass('active');

						break;
					case 'toggle_custom_mask':
						if (value == 'custom') {
							$(target).removeClass('mf_hidden');
						} else {
							$(target).addClass('mf_hidden');
						}
						break;
					case 'toggle_associated':
						/*
						* Helper data attributes
						* (data-toggle-parent = The selector for the that hold the targeted elements)
						* (data-toggle-hide = The selector for the elements to be hid)
						* (data-toggle-show = The selector for the elements to be shown)
						*/

						mfSingle.toggleAssociatedElements(undefined, element);
						break;
					case 'prevent_switch_off':
						$(element).prop('checked', true);
						mfSingle.showSnackbar({ text: 'message' in args ? args.message : localizedVars.text.removeInputN, customClass: 'warning' });
				}

			}
		},
		insertPreDefinedNodes: function (e, element, action, targetParent, value, currentField, targetSelector) {

			// These hard-coded elements are based off of the templates located at: common\templates\input
			switch (action) {
				case 'update_label':
					$(targetParent).find('label').prepend('<span class="mf_label">' + value + '</span>');
					break;
				case 'update_desc':
					var desc_position = $(element).closest('.field_container').find('#field_desc_position_mfield_' + currentField + '').getVal();
					if (desc_position == 'top') {
						if (targetSelector == '.mf-section-desc') {
							$(targetParent).find('.mf_input_container').before('<p class="mf-section-desc">' + value + '</p>');
						} else {
							$(targetParent).find('.mf_input_container').before('<span class="mf_description">' + value + '</span>');
						}

					} else {
						if (targetSelector == '.mf-section-desc') {
							$(targetParent).find('.mf_input_container').after('<p class="mf-section-desc">' + value + '</p>');
						} else {
							$(targetParent).find('.mf_input_container').after('<span class="mf_description">' + value + '</span>');
						}

					}
					break;
				case 'update_required':
					if (value) {
						$(targetParent).addClass('mfield_required');
						$(targetParent).find('.mf_field_container').children('label').append('<span class="mf_required">*</span>');
					}
					break;
			}

		},
		getAvailableContainers: function (e) {
			// Collect containers data into an array
			let containers = [];

			$('#mfields_holder').find(".mf_wrapper").each(function (index) {
				let container = {},
					columns = [],
					cols = $(this).find('.mf_col');

				$.map(cols, function (col, i) {
					let column = {},
						fields = [];

					$(col).find(".mfield").each(function (index) {
						fields.push($(this).attr('data-id'));
					});

					column['width'] = col.style.width;
					column['fields'] = fields;
					columns.push(column);
				});

				container['type'] = $(this).attr('data-container');
				container['columns'] = columns;
				containers.push(container);

			});
			return containers;
		},
		convertFormFieldsToArray: function (form, type) {
			// Convert form elements into an array
			let fields = [];
			if (type == 'fields') {
				let formID = mfSingle.formID;

				$(form).find(".single_field").each(function (i, item) {

					let field = {};
					field['id'] = $(item).attr('data-id');
					field['type'] = $(item).attr('data-type');
					field['formId'] = formID;

					let inputs = $(item).find(':input').serializeObject(),
						inputsObj = Object.assign({}, field, inputs); // Combine main field array with retrieved data

					fields.push(inputsObj);

				});
			} else if (type == 'settings') {

				$(form).find("li.mgform_settings").each(function (index) {
					let field = {};
					let data = $(this).find(':input').serializeArray();

					$.map(data, function (n, i) {

						field[n['name']] = n['value'];
					});
					fields.push(field);
				});

			} else if (type == 'actions') {
				var formID = mfSingle.formID;

				$(form).find(".single_action").each(function (i, item) {

					let action = {};
					action['id'] = $(item).attr('data-id');
					action['type'] = $(item).attr('data-type');
					action['formId'] = formID;

					let inputs = $(item).find(':input').serializeObject(),
						inputsObj = Object.assign({}, action, inputs); // Combine main field array with retrieved data

					fields.push(inputsObj);

				});
			}
			return fields;
		},
		checkFieldsEmptyCols: function (e) {
			$.each($('.mf_col'), function (e) {
				if ($(this).find('.single_field, .mg_field').length < 1) {
					$(this).addClass('mf-empty-col');
				} else {
					$(this).removeClass('mf-empty-col');
				}

			});
		},
		checkActionsEmptyCol: function (e) {
			if ($('.mf_action_col').find('.mfaction, .mg_action').length < 1) {
				$('.mf_action_col').addClass('mf-empty-col');
			} else {
				$('.mf_action_col').removeClass('mf-empty-col');
			}
		},
		showSnackbar: function (options) {
			Snackbar.show(options);
		},
		handleFieldDuplicate: function (e, el, current_field_id) {
			let new_field_id = mfSingle.incrementFieldID(),
				current_input_name = 'mform_' + mfSingle.formID + '_mfield_' + current_field_id + '',
				new_input_name = 'mform_' + mfSingle.formID + '_mfield_' + new_field_id + '',
				field_settings = $('.single_field_panel').find('[data-id="' + current_field_id + '"]'),
				field_preview = $(el).closest('.field_preview'),
				settingsClone = $(field_settings).clone(),
				inputClone = $(field_preview).clone();

			// Update preview IDs
			$(inputClone).attr('id', 'mf_' + mfSingle.formID + '_field_' + new_field_id + '');
			$(inputClone).attr('data-id', '' + new_field_id + '');
			let inputNameRegex = new RegExp(current_input_name, 'g');
			inputClone = $(inputClone)[0].outerHTML.replace(inputNameRegex, new_input_name);


			// Change field editor IDs and remove any active
			$(settingsClone).attr('data-id', '' + new_field_id + '');
			$(settingsClone).attr('id', 'mf_' + mfSingle.formID + '_field_' + new_field_id + '_editor');
			$(settingsClone).find('.field_form_id').text('(ID: ' + new_field_id + ')');
			let tempKeyRegex = new RegExp('_mfield_' + current_field_id + '', 'g');
			settingsClone = $(settingsClone)[0].outerHTML.replace(tempKeyRegex, '_mfield_' + new_field_id + '');

			// Remove editor jquery widget classes so it can be initiated again for this field
			$(settingsClone).find('.field_options').removeClass('ui-tabs');
			$(settingsClone).find('.mg-tabs li').removeClass('ui-tabs-active');
			$(settingsClone).find('.mg-tabs li').removeClass('ui-state-active');

			// Insert the element after changing all IDs
			$(inputClone).insertAfter(field_preview).addClass("mf-duplicate").removeClass('selected');
			$(settingsClone).insertAfter(field_settings).addClass("mf-duplicate").removeClass('is-active');

			mfSingle.initFieldsjQueryUI(false);
		},
		handleFieldDelete: function (e, el, current) {
			mfSingle.showSnackbar({
				text: localizedVars.text.removeFieldY,
				showButton: true,
				buttonText: localizedVars.text.yes,
				onButtonClick: function (element) {
					element.style.opacity = 0;

					if ($(el).closest('.field_preview').hasClass('selected')) {
						mfSingle.togglePanels(e, 'picker');
					}

					$(el).closest('.field_preview').fadeOut(300, function () {
						$(this).remove();
						mfSingle.checkFieldsEmptyCols();
						mfSingle.triggerConditionsRefresh('fields', 'remove');
					});
					$('.single_field_panel').find('[data-id="' + current + '"]').remove();
				},
			});
		},
		handleActionDelete: function (e, el, current) {
			mfSingle.showSnackbar({
				text: localizedVars.text.removeActionY,
				showButton: true,
				buttonText: localizedVars.text.yes,
				onButtonClick: function (element) {
					element.style.opacity = 0;

					$(el).closest('.single_action').fadeOut(300, function () {
						$(this).remove();
						mfSingle.checkActionsEmptyCol();
						mfSingle.triggerConditionsRefresh('actions', 'remove');
					});

				},
			});
		},
		handleContainerEdit: function (el, container) {
			alert('edit');
		},
		handleContainerDelete: function (el, container) {
			let wrappers = $('#mfields_holder').find('.mf_wrapper');
			if (wrappers.length === 1) {
				mfSingle.showSnackbar({ text: localizedVars.text.removeContainerN, customClass: 'warning' });
			} else {
				// Function to remove the container after deleting all child fields (If Yes clicked)
				mfSingle.showSnackbar({
					text: localizedVars.text.removeContainerY,
					showButton: true,
					buttonText: localizedVars.text.yes,
					onButtonClick: function (element) {
						element.style.opacity = 0;
						$(container).fadeOut(300, function () {

							$(this).find('.field_preview').each(function (i, obj) {
								let field_id = $(obj).attr('data-id');
								$(obj).remove();
								$('.single_field_panel').find('[data-id="' + field_id + '"]').remove();
							});

							$(this).remove();

						});
					},
				});
			}
		},
		handleContainerLayout: function (el, container) {
			// Get the template
			let count = $(container).find('.mfcol_preview').length,
				ctnControls = $(el).closest('.container_controls'),
				layoutTemplate = wp.template('container_layout'),
				layoutOutput = layoutTemplate({
					count: count,
				});

			// Add out element to the DOM and define any needed variables
			$(ctnControls).append(layoutOutput);
			let ctn_layout = document.getElementById('mf_ctn_layout'),
				count_slider = $(ctn_layout).find('.mf_column-count__slider'),
				count_number = $(ctn_layout).find('.mf_column-count__number'),
				count_save = $(ctn_layout).find('.mf_column-count__save');


			// Functions to handle different actions on the added element
			let finalizeThis = function (e) {

				$(el).removeClass('active');
				$(count_slider).unbind("input", changeHandler);
				$(count_save).unbind("click", saveHandler);
				$(el).unbind("click", removeHandler);
				$(container).unbind("mouseleave", removeHandler);

				$(ctn_layout).remove();
			};

			let removeHandler = function (e) {
				e.stopImmediatePropagation();
				e.stopPropagation();
				e.preventDefault();
				finalizeThis();
			};
			let saveHandler = function (e) {
				e.preventDefault();
				let newCount = parseInt($(count_slider).val());

				if (newCount !== count && newCount !== 0) {
					if (newCount > count) {
						// Handle adding columns
						let to_add = newCount - count,
							columnTemplate = wp.template('single_column'),
							columnOutput = columnTemplate({});

						for (var i = 0; i < to_add; i++) {
							$(container).append(columnOutput);
						}

						// Refresh sortables
						mfSingle.initContainersjQueryUI();

					} else {
						// Handle removing columns
						let to_remove = count - newCount,
							clones = [],
							i = 0;
						$($(container).find('.mfcol_preview').get().reverse()).each(function (e) {

							// Copy column fields into an array and remove the column
							if (i == to_remove) {
								// If we are done removing columns, append the saved fields to the first one
								for (var n = 0; n < clones.length; n++) {
									$(this).append(clones[n]);
								}
								return false;
							}

							$(this).find('.field_preview').each(function (e) {
								clones.unshift($(this));
							});

							$(this).remove();
							i++;
						});



					}
					mfSingle.checkFieldsEmptyCols();
				}

				finalizeThis();
			};
			let changeHandler = function (e) {
				$(this).next(count_number).html(this.value);
			};

			// Handle save/change/close actions on th element
			$(count_slider).bind("input", changeHandler);
			$(count_save).bind("click", saveHandler);
			$(el).bind("click", removeHandler);
			$(container).bind("mouseleave", removeHandler);

			// Show the element
			$(el).addClass('active');
			$(ctn_layout).show();
		},
		triggerConditionsRefresh: function (form, action) {

			let event = new CustomEvent("mfConditionalFormUpdated", { "detail": { "action": action } });
			switch (form) {
				case "fields":
					document.getElementById('mform_fields').dispatchEvent(event);
					break;
				case "settings":
					document.getElementById('mform_settings').dispatchEvent(event);
					break;
				case "actions":
					document.getElementById('mform_actions').dispatchEvent(event);
					break;
			}

		},
	};

	/**
	 * Only initialize after page load
	 */
	$(window).load(function () {
		mfSingle.onResize();
		mfSingle.init();
	});

	$(window).resize(function () {
		mfSingle.onResize();
	});

	/**
	 * Helper custom jQuery functions
	 */
	// Remove specific element class with only part of the class name using regex
	jQuery.fn.removeClassRegex = function (regex) {
		return $(this).removeClass(function (index, classes) {
			return classes.split(/\s+/).filter(function (c) {
				return regex.test(c);
			}).join(' ');
		});
	};
	// Get the correct input value with account to checkboxes and radios
	jQuery.fn.getVal = function () {

		switch ($(this).attr('type')) {
			case "radio":
				if ($(this).is(':checked')) {
					return $(this).val();
				} else {
					return false;
				}
			case "checkbox":
				if ($(this).is(':checked')) {
					return $(this).val();
				} else {
					return false;
				}
		}

		return $(this).val();

	};
	// recursive function that serializes inputs with arrays ( eg: input[x][z][y] ), it also accounts for empty brackets (checkboxes and multiselect)
	jQuery.fn.serializeObject = function () {
		let data = {},
			inputs = this.serializeArray();

		function buildInputObject(arr, val) {

			if (arr.length < 1) {
				return val;
			}

			let objkey = arr[0];
			if (objkey.slice(-1) == "]") {
				objkey = objkey.slice(0, -1);
			}

			let result = {};
			if (arr.length == 1) {
				result[objkey] = val;
			} else {
				arr.shift();
				let nestedVal = buildInputObject(arr, val);
				result[objkey] = nestedVal;

			}

			return result;
		}

		jQuery.each(inputs, function () {
			let val = this.value;
			let name = this.name.split("[");
			let object = buildInputObject(name, val);

			jQuery.extend(true, data, object);
		});

		return data;
	};

})(jQuery, megaForms, mfVars);
