(function ($, mf, localizedVars) {
	"use strict";
	const mfSingle = {
		entryID: 0,
		/**
	   * Function to initialize all necessary actions associated with current object.
	   */
		init: function () {
			// Define entry ID
			this.entryID = this.getentryID();

			// Initialize event listeners (controling all the different action across editor view, eg: click events)
			this.initEntryjQueryUI();
			this.initGeneralEvents();

			// Complete initialization
			this.initComplete();

		},
		/**
		 * This functions is responsible for initializeing any jQuery UI plugins associated with one or more entry elements.
		 */
		initEntryjQueryUI: function () {
			$('.mf-entry-accordions').accordion({
				heightStyle: "content",
				active: 0,
				collapsible: true,
			});

		},
		/**
		 * This functions is responsible for initializeing the event listeners associated of different compopnents of current page.
		 */
		initGeneralEvents: function () {
			// Switch full screen mode on and off
			$("#fullscreenbtn").click(this.switchFullScreen);
			// Show and hide empty fields
			$('.mf_entry_show_hidden input').change(this.toggleEmptyFields);
			// Handle notes insertion and deletion
			$("#mf_entry_save_note").click(this.addEntryNote);
			$('#mf_entry_sidebar').on("click", ".mf-entry-note-delete", function (e) {
				e.preventDefault();
				mfSingle.deleteEntryNote(e);
			});

			// Toggle editor view and values views
			$(".mf-entry-action-edit").on("click", function (e) {
				e.preventDefault();
				mfSingle.showEntryEditor(e);
			});
			$(".mf-entry-action-view").on("click", function (e) {
				e.preventDefault();
				mfSingle.hideEntryEditor(e);
			});
			// Handle saving entry changes
			$("#mfentry_save").click(this.saveEntry);
		},

		/**
	   * Complete initialization.
	   */
		initComplete: function (e) {
			// Remove initilization class to allow some element to display after initilization
			$('#mf_entry_sidebar').removeClass("mf_hidden");
			$('.initializing').removeClass("initializing");
		},
		/**
	   * Event callbacks.
	   */
		switchFullScreen: function (e) {
			e.preventDefault();
			$(window).scrollTop(0);
			$('.wp-toolbar').stop().toggleClass('fullscreen');
			$('body').stop().toggleClass('fullscreen');
			$('#wpcontent').stop().toggleClass('fullscreen');
			$('#mega-forms').stop().toggleClass('fullscreen');
			$('#mega-forms-container').stop().toggleClass('fullscreen');
		},
		toggleEmptyFields: function (e) {

			if ($(this).is(':checked')) {
				$('.mf-entry-values-table').find('.mf_entry_empty_row').removeClass('mf_hidden');
			} else {
				$('.mf-entry-values-table').find('.mf_entry_empty_row').addClass('mf_hidden');
			}

		},
		deleteEntryNote: function (e) {

			mfSingle.showSnackbar({
				text: localizedVars.text.deleteNoticeY,
				showButton: true,
				buttonText: localizedVars.text.yes,
				onButtonClick: function (element) {
					element.style.opacity = 0;
					let id = $(e.target).closest('.mf-entry-single-note').attr('data-id');

					$('.mf-entry-single-note[data-id="' + id + '"]').remove();
					if ($('.mf-entry-notes-list').find('.mf-entry-single-note').length == 0) {
						$('.mf-entry-notes-list').find('.no-notes').removeClass('mf_hidden');
					}

					mfSingle.ajaxDeleteNote(id);
				},
			});

		},
		addEntryNote: function (e) {
			e.preventDefault();
			let note = $('[name="mf_entry_note"]').val();
			if (note.length > 0) {
				$(this).addClass('disabled');
				mfSingle.ajaxAddNote(note);
			} else {
				mfSingle.showSnackbar({ text: localizedVars.text.addNoteN });
			}

		},
		showEntryEditor: function (e) {
			$('#mfentry_save').removeClass('mf_hidden');
			$('.mf-entry-action-view').removeClass('mf_hidden');
			$('.mf_entry_row').find('.edit').removeClass('mf_hidden');


			$('.mf-entry-action-edit').addClass('mf_hidden');
			$('.mf_entry_row').find('.view').addClass('mf_hidden');

			$('.mf_entry_row.mf_entry_empty_row.mf_hidden').removeClass('mf_hidden');


		},
		hideEntryEditor: function (e) {
			$('#mfentry_save').addClass('mf_hidden');
			$('.mf-entry-action-view').addClass('mf_hidden');
			$('.mf_entry_row').find('.edit').addClass('mf_hidden');


			$('.mf-entry-action-edit').removeClass('mf_hidden');
			$('.mf_entry_row').find('.view').removeClass('mf_hidden');

			if( !$('input[name="mf_entry_show_hidden"]').is(':checked') ){
				$('.mf_entry_row.mf_entry_empty_row').addClass('mf_hidden');
			}
		},
		saveEntry: function (e) {
			e.preventDefault();

			let fields = mf.serializeForm('form.single-mega-form');

			mfSingle.ajaxSaveEntry(fields);
		},

		/**
	   * AJAX.
	   */

		ajaxAddNote: function (note) {

			let data = {
				note: note,
				entry_id: mfSingle.entryID,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'add_entry_note',
				success: function (data) {
					let content = '';
					content += '<div class="mf-entry-single-note" data-id="' + data.id + '">';
					content += '<span class="mf-entry-note-box">' + note + '</span>';
					content += '<span class="mf-entry-note-date">' + data.date + '<a href="#" class="mf-entry-note-delete">' + localizedVars.text.delete + '</a></span>';
					content += '</div>';

					$('.mf-entry-notes-list').find('.no-notes').addClass('mf_hidden');
					$('[name="mf_entry_note"]').val('');
					$('#mf_entry_save_note').removeClass('disabled');
					$('.mf-entry-notes-list').append(content);

				},
				fail: function (data) {
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},

		ajaxDeleteNote: function (note_id) {

			let data = {
				note_id: note_id,
				entry_id: mfSingle.entryID,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'delete_entry_note',
				fail: function (data) {
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},

		ajaxSaveEntry: function (fields) {
			$('#saving').show();
			$('#mfentry_save').addClass('disabled');

			let data = {
				entry_id: mfSingle.entryID,
				fields: fields,
			};

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'save_entry_changes',
				success: function (data) {
					$('#mfentry_save').removeClass('disabled');
					$('#saving').hide();
					// Display sucess message in a snackbar notice and reload the page
					mfSingle.showSnackbar({ text: data.message });
					location.reload();
				},
				fail: function (data) {

					$('#mfentry_save').removeClass('disabled');
					$('#saving').hide();

					$('.mf_entry_row').find('.edit').each(function (e) {
						let id = $(this).attr('data-id');

						$(this).find('.mf_notice').remove();
						if (id !== undefined && data.notices[id] !== undefined) {
							$(this).addClass('mf_input_error');
							$(this).prepend('<span class="mf_notice">' + data.notices[id] + '</span>');
						}

					});
					// Display error message in a snackbar notice
					mfSingle.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},

		/**
	   * Helper Functions.
	   */

		getentryID: function () {
			// Try the URL
			let entryID = null,
				tmp = [];
			location.search
				.substr(1)
				.split("&")
				.forEach(function (item) {
					tmp = item.split("=");
					if (tmp[0] === 'id') entryID = decodeURIComponent(tmp[1]);
				});

			// Try Form data
			if (entryID == null || entryID == '' || isNaN(entryID)) {
				entryID = $('#mf_entry_panel').attr("data-id");
			}
			return parseInt(entryID);
		},
		showSnackbar: function (options) {
			Snackbar.show(options);
		},

	};

	/**
	 * Only initialize after page load
	 */
	$(window).load(function () {
		mfSingle.init();
	});

})(jQuery, megaForms, mfVars);
