(function ($, mf, localizedVars) {
	"use strict";
	const mfAll = {
		/**
	   * Function to initialize all necessary actions associated with current object.
	   */
		init: function () {

			let context = mfAll.getContext();

			// Show tooltip
			$('[data-hook="show-tooltip"]').click(this.showTooltip);

			if (context == 'mf_forms' || context == 'mf_form_entries') {
				// Handle Bulk Actions
				$('input#doaction').click(this.handleBulkAction);
				// Handle Single Actions
				$('[data-hook="handle-item-action"]').click(this.handleSingleFormAction);
				// Expand single action buttons
				$('[data-hook="expand-form-action"]').click(this.expandFromActions);
			}

			if (context == 'mf_forms') {
				// Activate and deactivate with switch button (change form status)
				$('[data-hook="change-form-status"]').change(this.changeFormStatus);
				// Trigger new modal on 'add form' button click
				$('[data-hook="show-form-modal"]').click(this.createFormModal);
			}

			if (context == 'mf_entries' || context == 'mf_form_entries') {
				// Add and remove entry star
				$('[data-hook="change-entry-star"]').click(this.changeEntryStar);
				// Allow user to change default table headers
				$('[data-hook="show-entry-settings-modal"]').click(this.createEntriesSettingsModal);
			}

			if (context == 'mf_settings') {
				// Switch between setting tabs in the settings page
				$('[data-hook="toggle-tab"]').click(this.showTab);
				// Only initialize color picker if the associated library exists.
				if (typeof $.fn.wpColorPicker !== 'undefined') {
					$('.mf-color-field').wpColorPicker();
				}
				// Expand single action buttons
				$('[data-hook="save-settings"]').click(this.saveSettings);
				// Confirmation message for the uninstall input
				$('input#uninstall').click(this.uninstallConfirmation);
			}

			if (context == 'mf_import_export') {
				// Switch between tabs in the import/export page
				$('[data-hook="toggle-tab"]').click(this.showTab);
				$('input[name="import_mf_forms"]').click(this.importForms);

			}

		},
		/**
	   * Event callbacks.
	   */
		expandFromActions: function (e) {
			e.preventDefault();
			$(this).parent().stop().toggleClass('active');

			$(this).parent().closest('tr').mouseleave(function () {
				$(this).find('[data-hook="expand-form-action"]').parent().removeClass("active");
			});

		},
		changeFormStatus: function (e) {
			let form_id = this.value,
				is_active = 0;

			if ($(this).is(':checked')) {
				is_active = 1;
			}
			mfAll.ajaxChangeStatus({ form_id: form_id, is_active: is_active });

		},
		createFormModal: function (e) {
			e.preventDefault();

			let modalTemplate = wp.template('create-form-modal'),
				modalOutput = modalTemplate({
					title: localizedVars.labels.create,
					label: localizedVars.labels.formName,
					spinner: localizedVars.spinner,
				});

			// Add out element to the DOM and add focus to the first input
			$('body').append(modalOutput).find('[data-hook="modal-title-input"]').focus();

			let modal = document.getElementById('mgform_modal');

			// Close Modal on close button click
			$('[data-hook="modal-close"]').click(function (e) {
				e.preventDefault();
				$(modal).remove();
			});

			// Trigger form creation on button click or keypress
			$('[data-hook="modal-title-input"]').keydown(function (e) {
				let code = e.keyCode || e.which;
				if (code == 13) {
					e.preventDefault();
				}
			});

			$('[data-hook="modal-title-input"]').keyup(function (e) {
				let code = e.keyCode || e.which;
				console.log(code);
				if (code == 13) {
					e.preventDefault();
					let name = $('[data-hook="modal-title-input"]').val();
					if (name) {
						mfAll.createNewMegaForm(name);
					} else {
						console.log('triggered');
						$(this).closest('.mfield').siblings('.error').text(localizedVars.labels.formNameError);
					}
				}
			});

			$('[data-hook="create-form"]').click(function (e) {
				e.preventDefault();
				let name = $('[data-hook="modal-title-input"]').val() ? $('[data-hook="modal-title-input"]').val() : $(this).attr('data-template-title');
				if (name) {
					let parent = $(this).attr('data-template-parent'),
						type = $(this).attr('data-template-type');
					mfAll.createNewMegaForm(name, type, parent);
				} else {
					$(this).closest('form').find('.error').text(localizedVars.labels.formNameError);
				}

			});

		},
		createNewMegaForm: function (name, type = 'blank', parent = 'main') {

			$('[data-hook="modal-title-input"]').prop('disabled', true);
			$('[data-hook="modal-title-submit"]').prop('disabled', true);
			$('[data-hook="modal-response"]').show();

			let data = {
				form_name: name,
				template_type: type,
				template_parent: parent,
			};

			mfAll.ajaxCreateNewForm(data);

		},
		handleBulkAction: function (e) {

			let action = $('select#bulk-action-selector-top').val();

			if (action == 'delete_entries') {
				e.preventDefault();
				mfAll.showSnackbar({
					text: localizedVars.labels.deleteEntriesY,
					showButton: true,
					buttonText: localizedVars.labels.yes,
					onButtonClick: function (element) {
						element.style.opacity = 0;
						$("form#mega_forms_list").submit();
					},
				});
			}

		},
		handleSingleFormAction: function (e) {
			e.preventDefault();
			let action = $(this).attr('data-action'),
				action_id = $(this).attr('data-action-id');
			$('[name="single_action"]').val(action);
			$('[name="single_action_id"]').val(action_id);

			$("form#mega_forms_list").submit();
		},
		changeEntryStar: function (e) {
			let entry_id = this.value,
				is_starred = 0;

			if ($(this).is(':checked')) {
				is_starred = 1;
			}
			mfAll.ajaxChangeEntryStar({ entry_id: entry_id, is_starred: is_starred });

		},
		createEntriesSettingsModal: function (e) {
			e.preventDefault();
			let content = '',
				i = 0,
				table_cols = $('.mf_form_entries_page').find('.wp-list-table thead th'),
				allFields = [];


			// Build field columns array
			$(table_cols).each(function (e) {
				let id = $(this).attr('id');
				if (id.indexOf("field_id_") >= 0) {
					let column_data = {
						id: id,
						label: $(this).text(),
						is_checked: !$(this).hasClass('hidden'),
					}
					allFields.push(column_data);
				}
			});

			// Build the markup
			content += '<form method="post">';
			content += '<table class="manage_hidden_fields_wrapper">';
			for (let n = 0; n < allFields.length; ++n) {
				// to create a new table row every 2 iterations
				i++;
				if (i == 1) {
					content += '<tr>';
				}
				let is_checked = allFields[n].is_checked ? 'checked="checked"' : '';

				content += '<td class="mf-field-inputs-switch">';
				content += '<label class="mfswitch mfswitch-size-small mfswitch-labelright">';
				content += '<input type="checkbox" name="mf_form_entry_hidden_cols[]" value="' + allFields[n].id + '" ' + is_checked + ' />';
				content += '<span class="mfswitch-slider round"></span>';
				content += '</label>';
				content += '<label for="mf_form_entry_hidden_cols[]" class="mf_label">' + allFields[n].label + '</label>';
				content += '</td>';

				if (i == 2) {
					i = 0;
					content += '</tr>';
				}
			}
			content += '</table>';
			content += '<input type="submit" id="submitButton" name="submitButton" value="' + localizedVars.labels.save + '" class="save_entry_hidden_cols button button-large button-primary">';
			content += '</form>';

			let modalTemplate = wp.template('empty-modal'),
				modalOutput = modalTemplate({
					heading: localizedVars.labels.changeColumns,
					desc: localizedVars.labels.changeColumnsDesc,
					content: content,
				});
			// Add out element to the DOM
			$('body').append(modalOutput);
			let modal = document.getElementById('mgform_modal');

			// Close Modal on close button click
			$('[data-hook="modal-close"]').click(function (e) {
				e.preventDefault();
				$(modal).remove();
			});

			// Trigger form creation on button click or keypress
			$('.save_entry_hidden_cols').click(function (e) {
				e.preventDefault();

				let form_id = $('#mega_forms_list').attr('data-id'),
					showingFields = [],
					hiddenFields = [];

				$('.manage_hidden_fields_wrapper').find(':checkbox:checked').each(function (i) {
					showingFields[i] = $(this).val();
				});

				$(allFields).each(function (i) {
					let column_id = allFields[i].id;
					if (jQuery.inArray(column_id, showingFields) == -1) {
						hiddenFields.push(allFields[i].id);
						// Hide column in the current view
						$('.column-' + column_id + '').addClass('hidden');
					} else {
						// Show column in the current view
						$('.column-' + column_id + '').removeClass('hidden');
					}
				});

				// Save preference to database
				if (typeof (form_id) !== undefined) {
					let hiddenCount = $(hiddenFields).length > 0 ? $(hiddenFields).length : 0;
					$('.fields_count').find('.hidden_fields_count').text(hiddenCount);
					mfAll.ajaxSaveEntryHiddenCols({ hidden_columns: hiddenFields, form_id: form_id });
				}

				$(modal).remove();
			});

		},
		showTab: function (e) {
			e.preventDefault();
			let el = e.currentTarget,
				panelToDisplay = $(el).data('panel'), // ID based
				panelToHide = $(el).data('disable');	// Class based

			// Hide active panels
			$(el).siblings('.active').removeClass('active');
			$('.' + panelToHide + '.active').removeClass('active');

			// Display the one selected
			$(el).addClass('active');
			$('#' + panelToDisplay + '').addClass('active');
		},
		showTooltip: function (e) {
			e.preventDefault();

			let el = e.currentTarget,
				content = $(el).find('#tooltip-content'),
				offest = 0;

			offest = ($(el).offset().top - $(el).closest('#mega-body-settings').offset().top) + 30;

			if (offest !== 0) {
				$(content).css({ top: '' + offest + 'px' }).fadeIn();
				$(el).mouseleave(function () {
					$(content).fadeOut();
				});
			}
		},
		saveSettings: function (e) {
			e.preventDefault();
			let settings = mf.serializeForm('form#megaforms_options');
			mfAll.ajaxSaveSettings({ settings: settings });
		},
		uninstallConfirmation: function (e) {
			let switchBtn = this;
			if ($(switchBtn).prop("checked") == true) {
				$(switchBtn).prop("checked", false);

				mfAll.showSnackbar({
					text: localizedVars.labels.uninstallY,
					showButton: true,
					buttonText: localizedVars.labels.yes,
					onButtonClick: function (element) {
						element.style.opacity = 0;
						$(switchBtn).prop("checked", true);
					},
				});
			}
		},
		importForms: function (e) {

			e.preventDefault();

			let formData = new FormData(),
				files = $('input#mf_import_file')[0].files;

			if (files.length == 0) {
				mfAll.showSnackbar({ text: localizedVars.labels.selectFile, customClass: 'failed' });
			} else if (files[0].type !== 'text/plain') {
				mfAll.showSnackbar({ text: localizedVars.labels.selectJsonFile, customClass: 'failed' });
			} else {
				formData.append('mf_import_file', files[0]);
				mfAll.ajaxImportForms(formData);
			}

		},
		/**
	   * Ajax methods.
	   */
		ajaxChangeStatus: function (data) {

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'set_form_status',
				success: function (response) { /* Do nothing on the front end, status is changed */ },
				fail: function (data) {
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});

		},
		ajaxChangeEntryStar: function (data) {

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'set_entry_star',
				success: function (response) { /* Do nothing on the front end, star is changed */ },
				fail: function (data) {
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});

		},
		ajaxCreateNewForm: function (data) {

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'create_form',
				success: function (response) {

					let redirect = response.redirect;
					jQuery('[data-hook="modal-response"]').find('img').hide();

					if (redirect.length === 0) {
						jQuery('[data-hook="modal-response"]').find('span').append(response.message);
					} else {
						jQuery('[data-hook="modal-response"]').find('span').append(response.message);
						location.href = redirect;
					}

				},
				fail: function (data) {
					jQuery('#mgform_modal').remove();
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},
		ajaxSaveEntryHiddenCols: function (data) {
			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'save_form_hidden_entry_columns',
				success: function (response) { /* Do nothing on the front end, hidden cols are saved */ },
				fail: function (data) {
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},
		ajaxSaveSettings: function (data) {
			$('#saving').show();
			$('#mfsettings_save').addClass('disabled');

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'save_settings',
				success: function (response) {
					$('#mfsettings_save').removeClass('disabled');
					$('#saving').hide();
					mfAll.showSnackbar({ text: response.message });
				},
				fail: function (data) {
					$('#mfsettings_save').removeClass('disabled');
					$('#saving').hide();
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},
		ajaxImportForms: function (data) {
			$('#saving').show();
			$('input[name="import_mf_forms"]').addClass('disabled');

			mf.ajax({
				data: data,
				isadmin: true,
				callback: 'import_forms',
				success: function (response) {
					$('input#mf_import_file').val('');
					$('input[name="import_mf_forms"]').removeClass('disabled');
					$('#saving').hide();
					mfAll.showSnackbar({ text: response.message });
				},
				fail: function (data) {
					$('input#mf_import_file').val('');
					$('input[name="import_mf_forms"]').removeClass('disabled');
					$('#saving').hide();
					mfAll.showSnackbar({ text: data.message, customClass: 'failed' });
				}
			});
		},
		/**
	   * Helper Functions.
	   */
		showSnackbar: function (options) {
			Snackbar.show(options);
		},
		getContext: function () {
			if (localizedVars.context !== undefined) {
				return localizedVars.context;
			}
			return false;
		},
	};
	/**
	 * Only initialize after page load
	 */
	$(window).load(function () {
		mfAll.init();

	});
})(jQuery, megaForms, mfVars);
