<?php

/**
 * Render Mega Forms General View
 *
 * @link       https://wpali.com
 * @since      1.0.0
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/admin/partials/views/settings-views
 */
if (!defined('ABSPATH')) {
  exit; // Exit if accessed directly
}

if (!class_exists('MegaForms_Settings')) {
  require_once MEGAFORMS_ADMIN_PATH . 'partials/views/settings-view/mega-settings.php';
}

class MegaForms_Settings_Render extends MegaForms_Settings
{

  /**
   * Holds the setting option tabs available for megaforms
   *
   * @since   1.0.0
   * @var    array
   */
  public $option_tabs = array();
  /**
   * Holds the setting options available for megaforms
   *
   * @since   1.0.0
   * @var    array
   */
  public $options = array();

  /**
   * Constructor.
   */
  public function __construct()
  {

    // Define the available options and option tabs
    $this->option_tabs = self::get_option_tabs();
    $this->options = self::get_options();
    // Render
    $this->render();
  }

  /**
   * Render form settings on the backend.
   *
   * @since    1.0.0
   */
  private function render()
  {
?>
    <div class="megacontainer_row">
      <h2 class="mg_forms_label">
        <?php esc_html_e('Settings', 'megaforms'); ?>
      </h2>
      <a id="mfsettings_save" class="megaforms-top-button" href="#" data-hook="save-settings"><span class="save-text"><?php esc_html_e('Save Settings', 'megaforms') ?></span></a>
      <div class="clearfix"></div>
    </div>
    <div class="clearfix"></div>
    <div id="mega-body" class="metabox-holder columns-2">
      <div id="mega-settings-container" class="mega-container">
        <div id="megaforms_settings">
          <div class="clearfix"></div>
          <?php
          $option_tabs = $this->option_tabs;
          $n = 1;
          foreach ($option_tabs as $key => $label) {
            $is_active = $n === 1 ? ' active' : '';
          ?>
            <a href="#" title="<?php echo $label; ?>" data-panel="<?php echo $key; ?>_settings" data-disable="megaforms_options" class="mf-settings-panel-label<?php echo $is_active; ?>" target="_self" data-hook="toggle-tab"><span class="app-menu-text"><?php echo $label; ?></span></a>
          <?php
            $n++;
          }
          ?>
          <div class="clearfix"></div>
        </div>

      </div>
      <div id="mega-body-settings" class="mega-body-content">
        <form method="post" id="megaforms_options" class="megaforms-settings">
          <ul id="mgsettingssholder">
            <?php
            $tabs = $this->options;
            $i = 1;
            foreach ($tabs as $tab => $options) {
              $is_active = $i === 1 ? ' active' : '';
            ?>
              <li id="<?php echo $tab; ?>_settings" class="megaforms_options<?php echo $is_active; ?>">
                <?php
                foreach ($options as $name => $args) {
                  $attrs = array();
                  $attrs['name'] = $name;
                  $attrs['id'] = $name;
                  if (isset($args['class'])) $attrs['class'] = $args['class'];
                  if (isset($args['placeholder'])) $attrs['placeholder'] = $args['placeholder'];
                  if (isset($args['value']) && ($args['type'] !== 'radio' && $args['type'] !== 'checkbox')) $attrs['value'] = esc_attr($args['value']);

                  if (isset($args['parent'])) {

                    $attrs['data-conditional-rules'] = json_encode(array(
                      'container' => '.sub-setting',
                      'rules' => array(
                        'name' => $args['parent'],
                        'operator' => 'is',
                        'value' => $args['parent_value'],
                      ),
                    ));
                ?>
                    <table class="mgform_<?php echo $tab; ?>_settings sub-setting" cellspacing="0" cellpadding="0">
                      <tbody>
                      <?php
                    } else {
                      ?>
                        <table class="mgform_<?php echo $tab; ?>_settings" cellspacing="0" cellpadding="0">
                          <tbody>
                          <?php
                        }
                          ?>
                          <tr>
                            <th><label for='<?php echo $name ?>'></label><?php echo $args['label'] ?>
                              <?php
                              if (isset($args['desc'])) {
                                echo $this->get_description_tip_markup($args['label'], $args['desc']);
                              }
                              ?>
                            </th>
                            <td class='mf-inner-field'>
                              <?php
                              if (isset($args['before_field'])) {
                                echo $args['before_field'];
                              }
                              ?>
                              <?php switch ($args['type']) {
                                case 'switch':
                                  if (mfget_bool_value($args['value'])) $attrs['checked'] = 'checked';
                                  echo get_mf_switch($attrs);
                                  break;
                                case 'textarea':
                                  echo get_mf_textarea($attrs);
                                  break;
                                case 'radio':
                                case 'checkbox':
                                  if (isset($args['options'])) {
                                    foreach ($args['options'] as $key => $title) {
                                      $attrs['id'] = $attrs['id'] . '_' . $key;
                                      $attrs['value'] = $key;
                                      if ($args['value'] == $key) $attrs['checked'] = 'checked';
                                      echo get_mf_input($args['type'], $attrs);
                                      if (isset($attrs['checked']) == $key) unset($attrs['checked']);
                              ?>
                                      <label for="<?php echo $attrs['id']; ?>"><?php echo $title; ?></label>
                              <?php
                                    }
                                  }
                                  unset($attrs['value']);
                                  break;
                                case 'select':
                                  if (isset($args['options'])) {
                                    echo get_mf_select($attrs, $args['options']);
                                  }
                                  break;
                                case 'custom':
                                  if (isset($args['content'])) {
                                    echo $args['content'];
                                  }
                                  break;
                                default:
                                  echo get_mf_input($args['type'], $attrs);
                                  break;
                              } ?>
                              <?php
                              if (isset($args['after_field'])) {
                                echo $args['after_field'];
                              }
                              ?>
                            </td>
                          </tr>
                          </tbody>
                        </table>
                      <?php
                    }
                      ?>
              </li>
            <?php
              $i++;
            }
            ?>
          </ul>
        </form>
      </div>
      <div id="saving" style="display: none;"></div>
    </div>
    <div class="clearfix"></div>
<?php
  }
}
