<?php

/**
 * Render Mega Forms General View
 *
 * @link       https://wpali.com
 * @since      1.0.0
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/admin/partials/views/settings-views
 */
if (!defined('ABSPATH')) {
  exit; // Exit if accessed directly
}
class MegaForms_Settings
{

  /**
   * Returns the default settings
   *
   */
  public static function get_option_tabs()
  {

    return apply_filters('megaforms_option_tabs', array(
      'general'    => __('General', 'megaforms'),
      'email'      => __('Emails', 'megaforms'),
      'validation' => __('Validation Messages', 'megaforms'),
      'misc'       => __('Misc', 'megaforms'),
    ));
  }
  /**
   * Returns the callbacks for all the options per each tab.
   * Each callback must return an array that contains the following: ( `key`, `label`, `description (optional)`, `content` )
   * Each option is assigned a value representing the type of data of that option
   *
   * @return array
   */
  public static function get_options()
  {

    $validation_messages = mf_api()->get_validation_messages();
    
    # Assign callback functions to each tab ( a callback for every option )
    $options = array(
      'general' => array(
        'load_form_styling' => array(
          'type' => 'switch',
          'label' => __('Load Form Styling', 'megaforms'),
          'desc' => __('Switching this off will stop all fields and containers related CSS from loading. Only switch off if you\'re going to load your custom CSS.', 'megaforms'),
          'value' => mfget_option('load_form_styling', true),
          'sanitization' => 'boolean',
        ),
        'storing_user_details' => array(
          'type' => 'switch',
          'label' => __('Store User Details', 'megaforms'),
          'desc' => __(' Switch this off to disable storing the IP address and User Agent on all forms. This helps with GDPR compliance.', 'megaforms'),
          'value' => mfget_option('storing_user_details', true),
          'sanitization' => 'boolean',
        ),
      ),
      'email' => array(
        'email_from_name' => array(
          'type' => 'text',
          'label' => __('From Name', 'megaforms'),
          'desc' => __('Type the default sender name for all of your outgoing MegaForms emails.', 'megaforms'),
          'value' => mfget_option('email_from_name', get_option('blogname')),
          'sanitization' => 'string',
        ),
        'email_from_address' => array(
          'type' => 'email',
          'label' => __('From Address', 'megaforms'),
          'desc' => __('Type the default sender email address for all of your outgoing MegaForms emails.', 'megaforms'),
          'value' => mfget_option('email_from_address', get_option('admin_email')),
          'sanitization' => 'email',
        ),
        'email_template' => array(
          'type' => 'custom',
          'label' => __('Template', 'megaforms'),
          'desc' => __('Select which format you want for all of your outgoing MegaForms emails.', 'megaforms'),
          'value' => mfget_option('email_template', 'html'),
          'content' => '<label for="email_template[1]" class="option-html"><span class="mega-icons-document-list"></span><input type="radio" id="email_template[1]" name="email_template" value="html" ' . checked(mfget_option('email_template', 'html'), 'html', false) . '>' . __('HTML Template', 'megaforms') . '</label><label for="email_template[2]" class="option-none"><span class="mega-icons-document-text"></span><input type="radio" id="email_template[2]" name="email_template" value="none" ' . checked(mfget_option('email_template', 'html'), 'none', false) . '>' . __('Plain Template', 'megaforms') . '</label>',
          'sanitization' => 'string',
        ),
        'email_header_image' => array(
          'type' => 'url',
          'label' => __('Header Image', 'megaforms'),
          'desc' => __('URL to the logo you want to be displayed at the top of all your outgoing MegaForms emails.', 'megaforms'),
          'value' => mfget_option('email_header_image'),
          'sanitization' => 'url',
        ),
        'email_footer_text' => array(
          'type' => 'textarea',
          'label' => __('Footer Text', 'megaforms'),
          'desc' => __('Type the text to appear in the footer of all your outgoing MegaForms emails.', 'megaforms'),
          'value' => mfget_option('email_footer_text', '{mf:wp site_title}'),
          'sanitization' => 'string',
        ),
        'email_primary_text_color' => array(
          'type' => 'text',
          'label' => __('Primary Color', 'megaforms'),
          'desc' => __('The primary text color for MegaForms email templates.', 'megaforms'),
          'value' => mfget_option('email_primary_text_color', '#000000'),
          'class' => 'mf-color-field',
          'sanitization' => 'color',
        ),
        'email_secondary_text_color' => array(
          'type' => 'text',
          'label' => __('Secondary Color', 'megaforms'),
          'desc' => __('The secondary text color for MegaForms email templates.', 'megaforms'),
          'value' => mfget_option('email_secondary_text_color', '#cccccc'),
          'class' => 'mf-color-field',
          'sanitization' => 'color',
        ),
        'email_primary_bg_color' => array(
          'type' => 'text',
          'label' => __('Primary Bg Color', 'megaforms'),
          'desc' => __('The primary background color for MegaForms email templates.', 'megaforms'),
          'value' => mfget_option('email_primary_bg_color', '#f3f3f5'),
          'class' => 'mf-color-field',
          'sanitization' => 'color',
        ),
        'email_secondary_bg_color' => array(
          'type' => 'text',
          'label' => __('Secondary Bg Color', 'megaforms'),
          'desc' => __('The secondary background color for MegaForms email templates.', 'megaforms'),
          'value' => mfget_option('email_secondary_bg_color', '#ffffff'),
          'class' => 'mf-color-field',
          'sanitization' => 'color',
        ),
      ),
      'validation' => array(
        'timetrap_error' => array(
          'type' => 'text',
          'label' => __('Timetrap Error', 'megaforms'),
          'desc' => __('Type the error message for spam entries detected by Timetrap.', 'megaforms'),
          'placeholder' => mfget('form_validation_timetrap_error', $validation_messages),
          'value' => mfget_option('timetrap_error'),
          'sanitization' => 'string',
        ),
        'honeypot_error' => array(
          'type' => 'text',
          'label' => __('Honeypot Error', 'megaforms'),
          'desc' => __('Type the error message for spam entries detected by Honeypot trap.', 'megaforms'),
          'placeholder' => mfget('form_validation_honeypot_error', $validation_messages),
          'value' => mfget_option('honeypot_error'),
          'sanitization' => 'string',
        ),
        'required_notice' => array(
          'type' => 'text',
          'label' => __('Required Field', 'megaforms'),
          'desc' => __('Type the error message for required single entry fields.', 'megaforms'),
          'placeholder' => mf_api()->get_validation_required_notice(),
          'value' => mfget_option('required_notice'),
          'sanitization' => 'string',
        ),
        'compound_required_notice' => array(
          'type' => 'text',
          'label' => __('Required Fields', 'megaforms'),
          'desc' => __('Type the error message for required multiple entry fields (e.g. name, email, password ...etc).', 'megaforms'),
          'placeholder' => mf_api()->get_validation_compound_required_notice(),
          'value' => mfget_option('compound_required_notice'),
          'sanitization' => 'string',
        ),
        'options_required_notice' => array(
          'type' => 'text',
          'label' => __('Required Option', 'megaforms'),
          'desc' => __('Type the error message for required option fields (e.g. radios, checkboxes, selectboxes).', 'megaforms'),
          'placeholder' => mf_api()->get_validation_required_notice(array('type' => 'choice')),
          'value' => mfget_option('options_required_notice'),
          'sanitization' => 'string',
        ),
        'address_required_notice' => array(
          'type' => 'text',
          'label' => __('Required Address', 'megaforms'),
          'desc' => __('Type the error message for required address field.', 'megaforms'),
          'placeholder' => mf_api()->get_validation_compound_required_notice(array('type' => 'address')),
          'value' => mfget_option('address_required_notice'),
          'sanitization' => 'string',
        ),
        'email_validation' => array(
          'type' => 'text',
          'label' => __('Invalid Email', 'megaforms'),
          'desc' => __('Type the error message for invalid email address.', 'megaforms'),
          'placeholder' => __('Please enter a valid email.', 'megaforms'),
          'value' => mfget_option('email_validation'),
          'sanitization' => 'string',
        ),
        'website_validation' => array(
          'type' => 'text',
          'label' => __('Invalid Website', 'megaforms'),
          'desc' => __('Type the error message for invalid web address.', 'megaforms'),
          'placeholder' => __('Please enter a valid url.', 'megaforms'),
          'value' => mfget_option('website_validation'),
          'sanitization' => 'string',
        ),
        'date_validation' => array(
          'type' => 'text',
          'label' => __('Invalid Date', 'megaforms'),
          'desc' => __('Type the error message for invalid date.', 'megaforms'),
          'placeholder' => __('Please enter a valid date.', 'megaforms'),
          'value' => mfget_option('date_validation'),
          'sanitization' => 'string',
        ),
        'date_range_validation' => array(
          'type' => 'text',
          'label' => __('Invalid Date Range', 'megaforms'),
          'desc' => __('Type the error message for invalid date range.', 'megaforms'),
          'placeholder' => __('Please enter a valid date range.', 'megaforms'),
          'value' => mfget_option('date_range_validation'),
          'sanitization' => 'string',
        ),
      ),
      'misc' => array(
        'uninstall' => array(
          'type' => 'switch',
          'label' => __('Uninstall', 'megaforms'),
          'desc' => __('Permanently delete all MegaForms data upon uninstallation.', 'megaforms'),
          'value' => mfget_option('uninstall', false),
          'sanitization' => 'boolean',
        ),
      ),
    );

    return apply_filters('megaforms_settings_options', $options);
  }
  /**
   * Update options into the database
   *
   * @return bool
   */
  public static function update_options($settings)
  {

    $available_options = self::get_options();
    $options = is_array($available_options) && !empty($available_options) ? call_user_func_array('array_merge', $available_options) : array();

    foreach ($options as $option_key => $option_args) {
      // Make sure switch/checkbox field values are also included
      if ($option_args['sanitization'] == 'boolean') {
        $value = mfget($option_key, $settings, false);
      } else {
        $value = mfget($option_key, $settings);
      }

      mfupdate_option($option_key, $value, $option_args['sanitization']);
    }

    return true;
  }

  /**
   *  Return description tooltip markup.
   *
   */
  protected function get_description_tip_markup($label, $content)
  {

    $html = '';
    $html .= '<span class="mf-tooltip mega-icons-question-circle" data-hook="show-tooltip">';
    $html .= '<span style="display:none;" id="tooltip-content"><h4>' . $label . '</h4>' . $content . '</span>';
    $html .= '</span>';

    return $html;
  }
}
