<?php

/**
 * @link       https://wpali.com
 * @since      1.0.3
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/actions
 */

/**
 * Load actions and handle registration
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/actions
 * @author     ALI KHALLAD <ali@wpali.com>
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class MF_Actions
{

	private static $actions = array();

	public static function register($action)
	{

		self::$actions[$action->type] = $action;
	}

	public static function get_single($type, $action = array())
	{

		if (isset(self::$actions[$type])) {
			/**
			 * Assign `action` property a value if it is provided.
			 *
			 * @param $action is required for some methods,
			 * failing to provide it will trigger an error
			 */

			# Clone the object before assigning property values 
			$action_object = clone self::$actions[$type];

			if (!empty($action)) {

				if ($action['formId']) {
					$action_object->form_id = !empty($action['formId']) ? absint($action['formId']) : 0;
					unset($action['formId']);
				}
				if ($action['id']) {
					$action_object->action_id = !empty($action['id']) ? absint($action['id']) : 0;
					unset($action['id']);
				}

				$action_object->action = is_array($action) ? $action : array();
			}

			return $action_object;
		}

		return false;
	}

	public static function get($type, $action = array())
	{
		return self::get_single($type, $action);
	}

	/**
	 * Get all MegaForms actions.
	 *
	 */
	public static function get_actions()
	{
		return self::$actions;
	}
}

require_once(MEGAFORMS_COMMON_PATH . 'partials/form/actions/class-mega-forms-action.php');
foreach (glob(MEGAFORMS_COMMON_PATH . 'partials/form/actions/class-mega-forms-action-*.php') as $action_filename) {
	require_once($action_filename);
}
