<?php

/**
 * @link       https://wpali.com
 * @since      1.0.3
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/fields
 */

/**
 * Load all fields and handle their registration
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/fields
 * @author     ALI KHALLAD <ali@wpali.com>
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class MF_Fields
{

	private static $fields = array();
	private static $jsDependencies = array();
	private static $cssDependencies = array();

	/**
	 * Register a new field.
	 *
	 */
	public static function register($field)
	{

		self::$fields[$field->type] = $field;

		if ($field->hasJSDependency) {
			self::$jsDependencies[$field->type] = $field->get_field_js_dependencies();
		}

		if ($field->hasCSSDependency) {
			self::$cssDependencies[$field->type] = $field->get_field_css_dependencies();
		}
	}
	/**
	 * Get a single fields fields.
	 *
	 */
	public static function get_single($type, $field = array())
	{

		if (isset(self::$fields[$type])) {

			/**
			 * Assign `field` property a value if it is provided.
			 *
			 * @param $field is required for some methods,
			 * failing to provide it will trigger an error
			 */

			# Clone the object before assigning property values 
			$field_object = clone self::$fields[$type];

			if (!empty($field)) {

				if ($field['formId']) {
					$field_object->form_id = !empty($field['formId']) ? absint($field['formId']) : 0;
					unset($field['formId']);
				}

				if ($field['id']) {
					$field_object->field_id 	= !empty($field['id']) ? absint($field['id']) : 0;
					unset($field['id']);
				}

				$field_object->field = is_array($field) ? $field : array();
			}

			return $field_object;
		}

		return false;
	}
	/**
	 * Get a single fields fields.
	 *
	 */
	public static function get($type, $field = array())
	{

		return self::get_single($type, $field);
	}

	/**
	 * Get all MegaForms fields.
	 *
	 */
	public static function get_fields()
	{

		return self::$fields;
	}

	/**
	 * Get field dependencies ( JS | CSS ) .
	 *
	 */
	public static function get_field_dependencies($depType, $fieldType = '')
	{

		$Deps = array();

		if ($depType == 'js') {
			$availableDependecies = self::$jsDependencies;
		} elseif ($depType == 'css') {
			$availableDependecies = self::$cssDependencies;
		}

		# If field type is defined, return the dependencies only for that field. Otherwise, return all dependencies.
		if ($fieldType !== '') {

			if (isset($availableDependecies[$fieldType])) {
				$Deps = $availableDependecies[$fieldType];
			}
		} else {

			foreach ($availableDependecies as $field) {
				$Deps[] = $field;
			}
		}

		return $Deps;
	}
}

/**
 * Require the base classes
 *
 * @since    1.0.0
 */
require_once(MEGAFORMS_COMMON_PATH . 'partials/form/fields/base/class-mega-forms-field.php');
foreach (glob(MEGAFORMS_COMMON_PATH . 'partials/form/fields/base/class-mega-forms-field-*.php') as $field_filename) {
	require_once($field_filename);
}
/**
 * Require available fields
 *
 * @since    1.0.0
 */
foreach (glob(MEGAFORMS_COMMON_PATH . 'partials/form/fields/class-mega-forms-field-*.php') as $field_filename) {
	require_once($field_filename);
}
