<?php

/**
 * This file is used to produce all helper functions related to this plugin.
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/fields
 * @author     ALI KHALLAD <ali@wpali.com>
 */

if ( !defined( 'ABSPATH' ) ) {
    exit;
    // Exit if accessed directly
}

if ( !function_exists( 'mfget_form' ) ) {
    function mfget_form( $form_id = 0 )
    {
        $_form = false;
        
        if ( isset( $GLOBALS['mf_form'] ) ) {
            $form = $GLOBALS['mf_form'];
            if ( is_object( $form ) && !empty($form->ID) ) {
                if ( $form_id > 0 && $form_id == $form->ID || $form_id == 0 ) {
                    $_form = $form;
                }
            }
        }
        
        if ( !$_form && $form_id > 0 ) {
            $_form = mf_api()->get_form( $form_id );
        }
        if ( !$_form ) {
            return null;
        }
        return $_form;
    }

}
if ( !function_exists( 'mfget_form_id' ) ) {
    function mfget_form_id()
    {
        $form = mfget_form_id();
        return ( !empty($form) ? $form->ID : false );
    }

}
if ( !function_exists( 'mfget_template_filename' ) ) {
    function mfget_template_filename( $parts, $template_name )
    {
        // Make sure the template directry is set correctly
        
        if ( is_array( $parts ) ) {
            $file_name_parts = $parts;
        } else {
            $file_name_parts = array( $parts );
        }
        
        // Add filename to the array and split everything to return correct path
        $file_name_parts[] = $template_name . '.php';
        return implode( '/', $file_name_parts );
    }

}
if ( !function_exists( 'mflocate_template' ) ) {
    function mflocate_template( $template_name, $args, $protected = false )
    {
        # Tempates folder name to check for in the theme
        $template_path = 'mega-forms';
        # Default templates path
        $default_path = MEGAFORMS_COMMON_PATH . 'partials/templates/';
        # Extract array element into variables to be used inside the template
        if ( !empty($args) && is_array( $args ) ) {
            extract( $args );
        }
        # Priority: Look for the passed template name within the theme. [only if the called template is not protected]
        $template = false;
        if ( !$protected ) {
            $template = locate_template( array( trailingslashit( $template_path ) . $template_name, $template_name ) );
        }
        # Get default template.
        if ( !$template ) {
            if ( file_exists( trailingslashit( $default_path ) . $template_name ) ) {
                $template = trailingslashit( $default_path ) . $template_name;
            }
        }
        $located = ( !$protected ? apply_filters(
            'mflocate_template',
            $template,
            $template_name,
            $template_path
        ) : $template );
        // Display the template if it exists
        
        if ( file_exists( $located ) ) {
            if ( !$protected ) {
                do_action(
                    'mf_before_template',
                    $template_name,
                    $template_path,
                    $located,
                    $args
                );
            }
            include $located;
            if ( !$protected ) {
                do_action(
                    'mf_after_template',
                    $template_name,
                    $template_path,
                    $located,
                    $args
                );
            }
        }
    
    }

}
if ( !function_exists( 'mflocate_template_html' ) ) {
    function mflocate_template_html( $template_name, $args = array(), $protected = false )
    {
        // Like mflocate_template, but returns the HTML instead of including the file directly.
        ob_start();
        mflocate_template( $template_name, $args, $protected );
        return ob_get_clean();
    }

}
if ( !function_exists( 'mfget_browser' ) ) {
    function mfget_browser()
    {
        $u_agent = $_SERVER['HTTP_USER_AGENT'];
        $bname = 'Unknown';
        $platform = 'Unknown';
        $version = "";
        //First get the platform?
        
        if ( preg_match( '/linux/i', $u_agent ) ) {
            $platform = 'linux';
        } elseif ( preg_match( '/macintosh|mac os x/i', $u_agent ) ) {
            $platform = 'mac';
        } elseif ( preg_match( '/windows|win32/i', $u_agent ) ) {
            $platform = 'windows';
        }
        
        // Next get the name of the useragent yes seperately and for good reason
        
        if ( preg_match( '/MSIE/i', $u_agent ) && !preg_match( '/Opera/i', $u_agent ) ) {
            $bname = 'Internet Explorer';
            $ub = "MSIE";
        } elseif ( preg_match( '/Firefox/i', $u_agent ) ) {
            $bname = 'Mozilla Firefox';
            $ub = "Firefox";
        } elseif ( preg_match( '/Chrome/i', $u_agent ) ) {
            $bname = 'Google Chrome';
            $ub = "Chrome";
        } elseif ( preg_match( '/Safari/i', $u_agent ) ) {
            $bname = 'Apple Safari';
            $ub = "Safari";
        } elseif ( preg_match( '/Opera/i', $u_agent ) ) {
            $bname = 'Opera';
            $ub = "Opera";
        } elseif ( preg_match( '/Netscape/i', $u_agent ) ) {
            $bname = 'Netscape';
            $ub = "Netscape";
        }
        
        // finally get the correct version number
        $known = array( 'Version', $ub, 'other' );
        $pattern = '#(?<browser>' . join( '|', $known ) . ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
        if ( !preg_match_all( $pattern, $u_agent, $matches ) ) {
            // we have no matching number just continue
        }
        // see how many we have
        $i = count( $matches['browser'] );
        
        if ( $i != 1 ) {
            //we will have two since we are not using 'other' argument yet
            //see if version is before or after the name
            
            if ( strripos( $u_agent, "Version" ) < strripos( $u_agent, $ub ) ) {
                $version = $matches['version'][0];
            } else {
                $version = $matches['version'][1];
            }
        
        } else {
            $version = $matches['version'][0];
        }
        
        // check if we have a number
        if ( $version == null || $version == "" ) {
            $version = "?";
        }
        return array(
            'userAgent' => $u_agent,
            'name'      => $bname,
            'version'   => $version,
            'platform'  => $platform,
            'pattern'   => $pattern,
        );
    }

}
if ( !function_exists( 'mfget_ip_address' ) ) {
    function mfget_ip_address()
    {
        
        if ( !empty($_SERVER['HTTP_CLIENT_IP']) ) {
            $ip_address = $_SERVER['HTTP_CLIENT_IP'];
        } elseif ( !empty($_SERVER['HTTP_X_FORWARDED_FOR']) ) {
            $ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip_address = $_SERVER['REMOTE_ADDR'];
        }
        
        return $ip_address;
    }

}
if ( !function_exists( 'mf_sanitize' ) ) {
    function mf_sanitize( $value, $type )
    {
        switch ( $type ) {
            case 'boolean':
                $value = mfget_bool_value( $value );
                break;
            case 'string':
                $value = sanitize_textarea_field( $value );
                break;
            case 'integer':
                $value = (int) $value;
                break;
            case 'email':
                $value = sanitize_email( $value );
                break;
            case 'url':
                $value = esc_url_raw( $value );
                break;
            case 'color':
                $value = sanitize_hex_color( $value );
                break;
            default:
                $value = sanitize_text_field( $value );
                break;
        }
        return $value;
    }

}
if ( !function_exists( 'mfget_option' ) ) {
    function mfget_option( $key, $fallback = '' )
    {
        $value = get_option( 'megaforms_' . $key, $fallback );
        return $value;
    }

}
if ( !function_exists( 'mfupdate_option' ) ) {
    function mfupdate_option( $key, $value, $type = '' )
    {
        $value = stripslashes_deep( mf_sanitize( $value, $type ) );
        return update_option( 'megaforms_' . $key, $value );
    }

}
if ( !function_exists( 'mfget_bool_value' ) ) {
    function mfget_bool_value( $value )
    {
        return filter_var( $value, FILTER_VALIDATE_BOOLEAN );
    }

}
if ( !function_exists( 'mfget' ) ) {
    function mfget( $name, $array = null, $fallback = '' )
    {
        
        if ( $array === null ) {
            # For GET values, strip all HTML tags before sending it back. ( Prevent XSS )
            $val = ( isset( $_GET[$name] ) ? wp_strip_all_tags( $_GET[$name] ) : $fallback );
        } else {
            $val = ( isset( $array[$name] ) ? $array[$name] : $fallback );
        }
        
        return $val;
    }

}
if ( !function_exists( 'mfpost' ) ) {
    function mfpost( $name, $array = null, $stripslashes = true )
    {
        
        if ( $array === null && isset( $_POST[$name] ) ) {
            return ( $stripslashes ? stripslashes_deep( $_POST[$name] ) : $_POST[$name] );
        } elseif ( is_array( $array ) && isset( $array[$name] ) ) {
            return ( $stripslashes ? stripslashes_deep( $array[$name] ) : $array[$name] );
        }
        
        return null;
    }

}
if ( !function_exists( 'mfinput' ) ) {
    function mfinput( $type, $args = array(), $is_editor = false )
    {
        # Bail out if necessary arguements are not available, or if the requested type doesn't exist.
        if ( empty($args['id']) && empty($args['name']) ) {
            return false;
        }
        /*
         * Field attributes
         */
        $attrs = array();
        # Name and ID attributes
        $attrs['name'] = ( !empty($args['name']) ? $args['name'] : $args['id'] );
        if ( isset( $args['id'] ) ) {
            $attrs['id'] = $args['id'];
        }
        # Value
        
        if ( isset( $args['value'] ) && $args['value'] !== '' && $args['value'] !== null ) {
            $attrs['value'] = $args['value'];
        } elseif ( !empty($args['default']) && $args['default'] !== '' && $args['default'] !== null ) {
            $attrs['value'] = $args['default'];
        }
        
        # Placeholder
        if ( isset( $args['placeholder'] ) && $args['placeholder'] !== '' ) {
            $attrs['placeholder'] = $args['placeholder'];
        }
        # CSS Atrributes
        if ( isset( $args['class'] ) && !empty($args['class']) ) {
            $attrs['class'] = $args['class'];
        }
        if ( isset( $args['style'] ) && !empty($args['style']) ) {
            $attrs['style'] = $args['style'];
        }
        # Required
        if ( isset( $args['required'] ) && $args['required'] == true ) {
            $attrs['required'] = 'required';
        }
        # Conditional logic attributes
        if ( isset( $args['conditional_rules'] ) ) {
            $attrs['data-conditional-rules'] = ( is_array( $args['conditional_rules'] ) ? json_encode( $args['conditional_rules'] ) : $args['conditional_rules'] );
        }
        # Attributes for JS helpers to assist with live preview and editor conditional logic ( Only for Editor Screen )
        
        if ( $is_editor ) {
            if ( isset( $args['onchange_edit'] ) ) {
                $attrs['data-edit'] = $args['onchange_edit'];
            }
            if ( isset( $args['onchange_preview'] ) ) {
                $attrs['data-preview'] = $args['onchange_preview'];
            }
        }
        
        # Additional Attributes
        if ( isset( $args['attributes'] ) && count( $args['attributes'] ) > 0 ) {
            $attrs = array_merge( $attrs, $args['attributes'] );
        }
        /*
         * Field Parameters
         */
        $params = array();
        $params['container_tag'] = ( isset( $args['container_tag'] ) ? $args['container_tag'] : 'div' );
        $params['container_class'] = ( isset( $args['container_class'] ) ? 'mf_field_container ' . $args['container_class'] : 'mf_field_container' );
        $params['label'] = ( isset( $args['label'] ) ? $args['label'] : '' );
        $params['desc'] = ( isset( $args['desc'] ) ? $args['desc'] : '' );
        $params['desc_position'] = ( isset( $args['desc_position'] ) ? $args['desc_position'] : 'bottom' );
        $params['wrapper_class'] = ( isset( $args['wrapper_class'] ) ? $args['wrapper_class'] : '' );
        $params['before_label'] = ( isset( $args['before_label'] ) ? $args['before_label'] : '' );
        $params['after_label'] = ( isset( $args['after_label'] ) ? $args['after_label'] : '' );
        $params['before_field'] = ( isset( $args['before_field'] ) ? $args['before_field'] : '' );
        $params['after_field'] = ( isset( $args['after_field'] ) ? $args['after_field'] : '' );
        $params['before_input'] = ( isset( $args['before_input'] ) ? $args['before_input'] : '' );
        $params['after_input'] = ( isset( $args['after_input'] ) ? $args['after_input'] : '' );
        $params['required'] = ( isset( $args['required'] ) && $args['required'] == true ? true : false );
        $params['attributes'] = $attrs;
        # Enable Inline Modal for editor fields only
        
        if ( $is_editor && isset( $args['inline_modal'] ) && count( $args['inline_modal'] ) > 0 ) {
            $params['after_input'] = $params['after_input'] . '<span class="mf-inline-modal-btn mega-icons-dots-three-horizontal" data-templates="' . esc_attr( join( ',', $args['inline_modal'] ) ) . '"></span>';
            $params['wrapper_class'] = $params['wrapper_class'] . ' has_inline_modal';
        }
        
        # Notices
        if ( isset( $args['notice'] ) && !empty($args['notice']) && $args['notice'] !== false ) {
            
            if ( $type == 'subcustom' ) {
                $params['after_input'] = $params['after_input'] . '<span class="mf_compound_notice">' . $args['notice'] . '</span>';
            } else {
                $params['after_input'] = $params['after_input'] . '<span class="mf_notice">' . $args['notice'] . '</span>';
            }
        
        }
        switch ( $type ) {
            case 'text':
                $params['inputType'] = ( isset( $args['inputType'] ) ? $args['inputType'] : 'text' );
                break;
            case 'checkbox':
            case 'radio':
            case 'select':
                $params['options'] = ( isset( $args['options'] ) && is_array( $args['options'] ) ? $args['options'] : array() );
                break;
            case 'switch':
                $params['size'] = ( isset( $args['size'] ) ? $args['size'] : '' );
                $params['labelRight'] = ( isset( $args['labelRight'] ) ? $args['labelRight'] : false );
                break;
            case 'custom':
                $params['content'] = ( isset( $args['content'] ) ? $args['content'] : '{MF_CUSTOM_INPUT}' );
                break;
            case 'subcustom':
                $params['content'] = ( isset( $args['content'] ) ? $args['content'] : '{MF_CUSTOM_INPUT}' );
                $params['wrapper_tag'] = ( isset( $args['wrapper_tag'] ) ? $args['wrapper_tag'] : 'span' );
                if ( isset( $args['notice_css_class'] ) && $args['notice_css_class'] ) {
                    $params['wrapper_class'] = $params['wrapper_class'] . ' mf_sub_input_error';
                }
                unset( $params['container_tag'] );
                unset( $params['container_class'] );
                break;
        }
        $template_name = mfget_template_filename( 'inputs', $type );
        $field = mflocate_template_html( $template_name, apply_filters( 'mf_input_args', $params ), $is_editor );
        return $field;
    }

}
if ( !function_exists( 'mf_esc_attr' ) ) {
    function mf_esc_attr( $attr_key, $attr_val )
    {
        // Don't trim the `value` attribute
        
        if ( is_string( $attr_val ) && $attr_key !== 'value' ) {
            $attr_val = trim( $attr_val );
        } elseif ( is_bool( $attr_val ) ) {
            $attr_val = ( $attr_val ? 1 : 0 );
        } elseif ( is_array( $attr_val ) || is_object( $attr_val ) ) {
            $attr_val = json_encode( $attr_val );
        }
        
        // return the markup.
        return sprintf( ' %s="%s"', esc_attr( $attr_key ), esc_attr( $attr_val ) );
    }

}
if ( !function_exists( 'mf_esc_attrs' ) ) {
    function mf_esc_attrs( $attrs )
    {
        $output = '';
        foreach ( $attrs as $key => $val ) {
            // get the markup.
            $output .= mf_esc_attr( $key, $val );
        }
        // return the markup
        return trim( $output );
    }

}
if ( !function_exists( 'get_mf_input' ) ) {
    function get_mf_input( $type, $attrs )
    {
        return sprintf( '<input type="%s" %s/>', $type, mf_esc_attrs( $attrs ) );
    }

}
if ( !function_exists( 'get_mf_textarea' ) ) {
    function get_mf_textarea( $attrs )
    {
        $value = '';
        
        if ( isset( $attrs['value'] ) ) {
            $value = $attrs['value'];
            unset( $attrs['value'] );
        }
        
        return sprintf( '<textarea %s>%s</textarea>', mf_esc_attrs( $attrs ), esc_textarea( $value ) );
    }

}
if ( !function_exists( 'get_mf_checkbox' ) ) {
    function get_mf_checkbox( $attrs, $multi = false )
    {
        if ( $multi ) {
            $attrs['name'] = $attrs['name'] . '[]';
        }
        return get_mf_input( 'checkbox', $attrs );
    }

}
if ( !function_exists( 'get_mf_radio' ) ) {
    function get_mf_radio( $attrs )
    {
        return get_mf_input( 'radio', $attrs );
    }

}
if ( !function_exists( 'get_mf_switch' ) ) {
    function get_mf_switch( $attrs )
    {
        $attrs['value'] = "1";
        return '<label class="mfswitch">' . get_mf_checkbox( $attrs ) . '<span class="mfswitch-slider round"></span></label>';
    }

}
if ( !function_exists( 'get_mf_select' ) ) {
    function get_mf_select( $attrs, $options )
    {
        $value = '';
        
        if ( isset( $attrs['value'] ) ) {
            $value = mfget( 'value', $attrs );
            unset( $attrs['value'] );
        }
        
        
        if ( isset( $attrs['placeholder'] ) ) {
            $placeholderAttrs = array(
                'value'    => '',
                'class'    => 'mf_select_placeholder',
                'disabled' => 'disabled',
            );
            if ( '' == $value ) {
                $placeholderAttrs['selected'] = 'selected';
            }
            $placeholder = sprintf( '<option %s>%s</option>', mf_esc_attrs( $placeholderAttrs ), esc_attr( $attrs['placeholder'] ) );
            unset( $attrs['placeholder'] );
        } else {
            $placeholder = '';
        }
        
        return sprintf(
            '<select %s>%s%s</select>',
            mf_esc_attrs( $attrs ),
            $placeholder,
            mf_select_walker( $options, $value )
        );
    }

}
if ( !function_exists( 'mf_select_walker' ) ) {
    function mf_select_walker( $options, $value )
    {
        $output = '';
        // Loop over the available options and add them to the output variable.
        foreach ( $options as $key => $label ) {
            
            if ( is_array( $label ) ) {
                $output .= sprintf( '<optgroup label="%s">%s</optgroup>', esc_attr( $key ), mf_select_walker( $label, $value ) );
            } else {
                $attrs = array();
                $attrs['value'] = $key;
                if ( (string) $key === (string) $value ) {
                    $attrs['selected'] = 'selected';
                }
                $output .= sprintf( '<option %s>%s</option>', mf_esc_attrs( $attrs ), esc_html( $label ) );
            }
        
        }
        return $output;
    }

}
if ( !function_exists( 'mf_fs' ) ) {
    // Create a helper function for easy SDK access.
    function mf_fs()
    {
        global  $mf_fs ;
        
        if ( !isset( $mf_fs ) ) {
            // Include Freemius SDK.
            require_once MEGAFORMS_COMMON_PATH . 'partials/libraries/licensing/start.php';
            $mf_fs = fs_dynamic_init( array(
                'id'             => '7439',
                'slug'           => 'mega-forms',
                'premium_slug'   => 'mega-forms-pro',
                'type'           => 'plugin',
                'public_key'     => 'pk_c41e3d0345c2fc9c18685fa76f7a2',
                'is_premium'     => false,
                'premium_suffix' => 'Pro',
                'has_addons'     => false,
                'has_paid_plans' => false,
                'menu'           => array(
                'slug'    => 'mega-forms',
                'contact' => false,
                'support' => false,
                'pricing' => false,
            ),
                'is_live'        => true,
            ) );
        }
        
        return $mf_fs;
    }

}