<?php

/**
 * @link       https://wpali.com
 * @since      1.0.0
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/public
 * @author     ALI KHALLAD <ali@wpali.com>
 */
if ( ! defined( 'ABSPATH' ) ) {
   exit; // Exit if accessed directly
}

class Mega_Forms_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->load_dependencies();

	}

	/**
	 * Load the required dependencies for this class.
	 *
	 * @since    1.0.0
	 */
	private function load_dependencies() {

    // Load Shorcode class ( Some shortcodes will load their own dependencies )
		require_once MEGAFORMS_PUBLIC_PATH . 'partials/class-mega-forms-public-shortcodes.php';
    // Load form view class which is used for the shortcode
    require_once MEGAFORMS_PUBLIC_PATH . 'partials/class-mega-forms-public-form-view.php';

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

    # Enqueue the public stylesheet
		wp_register_style( 'mf-public', plugin_dir_url( __FILE__ ) . 'assets/css/mega-forms-public.css', array('mf-common-css'), $this->version, 'all' );

    if( mfget_option( 'load_form_styling', true ) ){
      wp_enqueue_style( 'mf-public' );
    }

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_register_script( 'mf-public', plugin_dir_url( __FILE__ ) . 'assets/js/mega-forms-public.js', array( 'jquery', 'mf-common-js' ), $this->version, false );
    $mfFormVars = array(
      'labels' => array(
        'x' => __( 'Placeholder', 'megaforms' ),
      ),
    );
    wp_localize_script( 'mf-public', 'mfVars', $mfFormVars);
    # Note: Any field specific JS is loaded from the shortcode callback only when needed
	}

	/**
	 * Listen to form submissions and process them if there are any.
	 *
	 * @since    1.0.0
	 */
	public function listen() {

    if ( isset( $_POST['mform_submit'] ) && isset( $_POST['_mf_form_id'] ) && isset( $_POST['_mf_nonce'] ) ) {

      mf_submission()->start( $_POST['_mf_form_id'], $_POST );

      # Make any necessary redirections
      if( mf_submission()->is_empty() !== true && mf_submission()->success ){
        if( mf_submission()->redirect !== false ){
          wp_redirect( mf_submission()->redirect );
          exit;
        }
      }

    }

	}

}
